/*************************************************************************
* MediaCoder - The universal audio/video transcoder (command line version)
*
* Copyright (c) 2006 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. The name of the author may not be used to endorse or promote products
*    derived from this software without specific prior permission.
*
* THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <time.h>
#include "mccore.h"

#define VER_MAJOR 0
#define VER_MINOR 1
#define VER_REV 1

#define APPNAME "MediaCoder CLI"
#define DEF_HTTPD_PORT 8000

#define PT_INT 1
#define PT_STRING 2
#define PT_ENUM 3
#define PT_BOOL 4
#define PT_FLOAT 5
#define PT_PTR 6
#define PT_FILE 7
#define PT_DIR 8
#define PT_SEPERATOR 0xe
#define PT_CATALOG 0xf

int iQueueItemIndex;
int LoadTextFile(char* filename, char** buffer, int bufsize);
int RetrieveFileSize(char* filename);
int EventHandler(MC_EVENT event, void* arg);
static MC_STATE* mcstate;
static HANDLE hEvent;
static char apppath[MAX_PATH];


#define EVENT EventHandler

int GetAppPath(char *buf,int bufsize)
{
	char *p;
	p=buf+GetModuleFileName(GetModuleHandle(NULL),buf,bufsize);
	while(p>buf && *(--p)!='\\');
	*(++p)=0;
	return p-buf;
}

int ShutDownNT(int fPowerOff)
{
	HANDLE hProcessHandle,hTokenHandle;
	LUID tmpLuid;
	TOKEN_PRIVILEGES tkpNew;
	TOKEN_PRIVILEGES tkpPrevious;
	DWORD lBufferNeeded;

	hProcessHandle = GetCurrentProcess();
	if (!OpenProcessToken(hProcessHandle, TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hTokenHandle)) return -1;

	//Get the LUID for the shutdown privilege
	LookupPrivilegeValue("", "SeShutdownPrivilege", &tmpLuid);
	tkpNew.PrivilegeCount = 1; // One privilege to set
	tkpNew.Privileges[0].Luid = tmpLuid;
	tkpNew.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;

	// Enable the shutdown privilege in the access token of this process.
	lBufferNeeded = 0;
	AdjustTokenPrivileges(hTokenHandle, FALSE, &tkpNew, sizeof(tkpPrevious), &tkpPrevious, &lBufferNeeded);

	// Force a Reboot/Poweroff (no option to save files to cancel out)
	ExitWindowsEx(fPowerOff?(EWX_FORCE|EWX_SHUTDOWN|EWX_POWEROFF):(EWX_FORCE|EWX_SHUTDOWN), 0xffff);
	return 0;
}

char* SetFileExt(char* filename, char* fileext)
{
	char *p;
	p=strrchr(filename,'.');
	if (p)
		strcpy(p+1,fileext);
	else
		strcat(filename,fileext);
	return filename;
}

int EventHandler(MC_EVENT event, void* arg)
{
	switch (event) {
	case MC_MSG_ALERT:
		printf("Alert: %s\n", arg);
		break;
	case MC_MSG_POPUP:
	case MC_MSG_INFO:
		printf("%s", arg);
		break;
	case MC_JOB_START:
		printf("---------------------------- Job %d started ----------------------------\n", iQueueItemIndex + 1);
		break;
	case MC_JOB_FINISH:	{
		printf("---------------------------- Job %d finished ---------------------------\n", iQueueItemIndex + 1);
		printf("Total completed: %d%%\n", mcstate->stats.fileprocessed * 100 / mcstate->totalfiles);
		iQueueItemIndex++;
		} break;
	}
	return 0;
}

int AddItem(T_ITEM* item)
{
	printf("Adding file %s\n", item->filename);
	mcAddItem(item);
	mcstate->totalfiles++;
	return 0;
}

#ifndef STANDALONE
int LoadTextFile(char* filename, char** buffer, int bufsize)
{
	DWORD bytes;
	HANDLE hFile=CreateFile(filename,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
	int len;

	if (hFile==INVALID_HANDLE_VALUE) return -1;
	len=GetFileSize(hFile,0);
	if (len+1>bufsize) {
		bufsize=len+1;
		*buffer=(char*)realloc(*buffer, bufsize);
	}
	*(*buffer+len)=0;
    ReadFile(hFile, *buffer, len, &bytes, 0);
	CloseHandle(hFile);
	return bytes;
}

int RetrieveFileSize(char* filename)
{
	HANDLE hFile = CreateFile(filename,GENERIC_READ,FILE_SHARE_READ,0,OPEN_EXISTING,0,0);
	DWORD sizelo, sizehi;
	if (hFile==INVALID_HANDLE_VALUE) return -1;
	sizelo = GetFileSize(hFile,&sizehi);
	CloseHandle(hFile);
	return (sizehi << 22) | (sizelo >> 10);
}
#endif

int LoadQueue(char* filename)
{
	char *p, *nl;
	T_ITEM itemdata;
	char *text = 0;
	int count = 0;
	BOOL m3u = FALSE;
	int i;

	if (LoadTextFile(filename, &text, 0) <= 0) return -1;
	p = text;
	if (!strncmp(p, "#EXTM3U", 7)) {
		m3u = TRUE;
		p += 7;
	}
	memset(&itemdata, 0, sizeof(T_ITEM));
	for (; p; p = nl) {
		while (*(p++) != '\n');
		nl = strchr(p, '\r');
		if (!nl) nl = strchr(p, '\n');
		if (nl) *nl = 0;
		if (!*p) continue;
		if (m3u && *p == '#') {
			p++;
			if (!strncmp(p, "EXTINF:", 7)) {
				p = strchr(p + 7, ',');
				if (!p) continue;
			} else if (!strncmp(p, "EXTINF_FLAG:", 12)) {
				p += 12;
				itemdata.flags = atoi(p);
			} else if (!strncmp(p, "EXTINF_TAG:", 11)) {
				char **ptag[] = {&itemdata.info.tags.artist, &itemdata.info.tags.title, &itemdata.info.tags.genre, &itemdata.info.tags.comment, 0};
				char *s;
				itemdata.info.tags.valid = 1;
				p += 11;
				for (i = 0; ptag[i]; i++) {
					if (!(s = strchr(p, ';'))) break;
					*s = 0;
					if (*p) *ptag[i] = strdup(p);
					p = s + 1;
				}
				itemdata.info.tags.year = atoi(p);
				p = strchr(p, ';');
				if (!p) continue;
				itemdata.info.tags.track = atoi(++p);
				p = strchr(p, ';');
				if (!p) continue;
				itemdata.info.tags.trackcount = atoi(++p);
				p = strchr(p, ';');
				if (!p) continue;
			} else if (!strncmp(p, "EXTINF_TIME:", 12)) {
				p += 12;
				itemdata.info.begin = atoi(p);
				p = strchr(p, ';');
				if (!p) continue;
				itemdata.info.end = atoi(++p);
				p = strchr(p, ';');
				if (!p) continue;
				itemdata.info.track = atoi(++p);
				p = strchr(p, ';');
				if (!p) continue;
				itemdata.info.chapter = atoi(++p);
				p = strchr(p, ';');
				if (!p) continue;
				itemdata.info.discroot = strdup(++p);
			} else if (!strncmp(p, "EXTINF_FMT:", 11)) {
				p += 11;
				itemdata.flags |= IF_PRIVATE_FORMAT;
				if ((p = strtok(p, ";"))) itemdata.target.container = (CONTAINER_FORMAT)atoi(p);
				if ((p = strtok(0, ";"))) itemdata.target.audioformat = (AUDIO_CODEC)atoi(p);
				if ((p = strtok(0, ";"))) itemdata.target.videoformat = (VIDEO_CODEC)atoi(p);
				if ((p = strtok(0, ";"))) itemdata.target.videoenc = (VIDEO_ENCODER)atoi(p);
			}
			continue;
		}
		{
		T_ITEM* item = mcMakeItem(p);
		memcpy(item, &itemdata, sizeof(T_ITEM));
		memset(&itemdata, 0, sizeof(T_ITEM));
		item->filename = strdup(p);
		item->info.filesize = RetrieveFileSize(item->filename);
		item->info.filetype = mcIdentifyFileType(item->filename);
		AddItem(item);
		}
		count++;
	}
	free(text);
	return count;
}

int AddMultipleFiles(char* filenames)
{
	T_ITEM* item;
	int count = 0;
	char *p = filenames;
	do {
		char *s = p;
		if (*p == '\"') {
			s++;
			for (p++; *p && *p!= '\"'; p++);
			if (*p) {
				*(p++) = 0;
			}
		} else {
			for (; *p && *p!= ' '; p++);
		}
		item = mcMakeItem(s);
		AddItem(item);
		count++;
	} while (*(p++) == ' ');
	return count;
}

void ShowInfo()
{
	printf("%s Version %d.%d.%d\n",
		APPNAME, VER_MAJOR, VER_MINOR, VER_REV);
	printf("Based on %s\n", mcGetVersionInfo());
	printf("This is a free software written by Stanley Huang <reachme@citiz.net>. Enjoy!\n\n");
	printf("Notice: this utility is just written for fun and only transcodes audio\n\n");
}

void ShowHelp()
{
	printf("Usage: mc [options] file1 file2 ...\nOptions:\n\
  -o <dir>\t\tSet output directory\n\
  -format <arg>\tSet target format (arg=mp3/vorbis/aac/aacplus/wma/wav)\n\
  -srate <arg>\tSet target sampling rate (arg=sampling rate in Hz)\n\
  -bitrate <arg>\tSet target bit rate (arg=bit rate in KBps)\n\
  -httpd <port>\tStart built-in HTTP daemon to provide web-based UI\n\
  -listfile <file>\tLoad file names from a list file for transcoding\n\
  -logfile <file>\tLog to file\n\
  -listpref\t\tList path and type of all avaiable settings\n\n");
}

void ShowStats()
{
	int sec;
	mcInvalidiateStatsData();

	switch (mcstate->stage) {
	case ST_PROCESSING_AUDIO:
		sec = mcstate->stats.dwElapsed >> 10;
		printf("%3.1f%% completed | %d seconds elapsed", mcstate->stats.percent, sec);
		if (mcstate->stats.percent) {
			sec = (int)((mcstate->stats.dwElapsed / mcstate->stats.percent * (100 - mcstate->stats.percent)) / 1000);
			printf(" | %d seconds remain        ", sec + 1);
		}
		break;
	case ST_PROCESSING_VIDEO: {
		VIDEO_STATS* vstats = &mcstate->vstats;
		printf("%d:%02d\t%df\t%4.1ffps\t%dmins\t%dMB\t%dKbps/%dKbps",
			vstats->pos / 60,
			vstats->pos % 60,
			vstats->frame,
			vstats->fps,
			vstats->remaintime,
			vstats->estsize  >> 10,
			vstats->vbitrate,
			vstats->abitrate);
		} break;
	}
	putchar('\r');
}

int SigHandler() {
	printf("\nCaught signal. Shutting down...\n");
	return 0;
}

int main(int argc, char* argv[])
{
	char* fmt = 0;
	char* outdir;
	char* logfile = 0;
	char* cfgfile = 0;
	int optexit = 0;
	int bitrate = 128;
	int srate = 0;
	int httpd = 0;
	int i;
	HANDLE hThread;

	SetConsoleCtrlHandler( (PHANDLER_ROUTINE) SigHandler, 0);
	GetAppPath(apppath,sizeof(apppath));
	SetCurrentDirectory(apppath);
	outdir = apppath;

	ShowInfo();

	// initialize the library
	if (!(mcstate = mcInit(&EventHandler))) {
		printf("Error occured when initializing MediaCoder core. Exit.\n");
		return -1;
	}

	// parse command line options
	for (i = 1; i < argc; i++) {
		if (argv[i][0] == '-' || argv[i][0] == '/') {
			char *arg = argv[i] + 1;
			if (!strcmp(arg, "o") && i + 1 < argc) {
				outdir = argv[++i];
			} else if (!strcmp(arg, "format") && i + 1 < argc) {
				fmt = argv[++i];
			} else if (!strcmp(arg, "bitrate") && i + 1 < argc) {
				bitrate = atoi(argv[++i]);
			} else if (!strcmp(arg, "srate") && i + 1 < argc) {
				srate = atoi(argv[++i]);
			} else if (!strcmp(arg, "logfile") && i + 1 < argc) {
				logfile = argv[++i];
			} else if (!strcmp(arg, "listfile") && i + 1 < argc) {
				// load list from file
				LoadQueue(argv[++i]);
			} else if (!strcmp(arg, "cfgfile") && i + 1 < argc) {
				// load settings from file
				mcLoadParam(argv[++i], 0);
			} else if (!strcmp(arg, "httpd")) {
				if (i + 1 < argc) httpd = atoi(argv[++i]);
				if (!httpd) httpd = DEF_HTTPD_PORT;
			} else {
				if (arg[0] != '?' && !strstr(arg, "help")) {
					printf("\nInvalid parameters - %s\n", argv[i]);
				}
				ShowHelp();
				optexit = 1;
				break;
			}
		} else {
            T_ITEM* item = mcMakeItem(argv[i]);
			AddItem(item);
		}
	}

	if (!optexit && mcstate->totalfiles <= 0) {
		printf("No file in the queue\n");
		optexit = 1;
	}

	if (optexit && !httpd)
		goto exit;

	// set some common settings
	iQueueItemIndex = 0;
	mcSetPrefValueInt("overall.ui.quiet", 1);
	mcSetPrefValueInt("overall.task.Overwrite", 1);
	mcSetPrefValueInt("audiosrc.mplayer.dll", 1);
	mcSetPrefValueInt("overall.task.Timeout", 3000);
	if (logfile) {
		mcSetPrefValueInt("overall.generic.logging", 1);
		mcSetPrefValueStr("overall.generic.logfile", logfile);
	}
	if (httpd) {
		printf("\nStarting HTTP daemon on port %d\n", httpd);
		mcSetPrefValueInt("overall.generic.httpd", 1);
		mcSetPrefValueInt("overall.generic.httpdPort", httpd);
	}

	if (!httpd && mcstate->totalfiles == 0) {
		ShowHelp();
		printf("No input files specified\n");
		return -1;
	}

	// do post initialization of the library
	if (mcPostInit(0)) {
		printf("Error occurred during post initialization.");
		goto exit;
	}

	if (httpd) {
		printf("Press [Enter] to stop HTTP daemon and exit.");
		getchar();
		goto exit;
	}

	// currently only do audio transcoding
	mcSetPrefValueInt("overall.video.format", VC_NONE);
	if (outdir) {
		mcSetPrefValueStr("overall.task.OutputFolder", outdir);
	}
	if (srate) {
		mcSetPrefValueInt("AudioFilters/Resampling/SamplingRate", srate);
	}
	if (!fmt) {
		printf("Target format not specified. Use the default setting.\n");
	} else if (!strcmp(fmt, "mp3")) {
		mcSetPrefValueInt("overall.audio.encoder", AC_LAMEMP3);	
		mcSetPrefValueInt("audioenc.lame.bitrate", bitrate);
		mcSetPrefValueInt("audioenc.lame.mode", 1);
	} else if (!strcmp(fmt, "vorbis")) {
		mcSetPrefValueInt("overall.audio.encoder", AC_VORBIS);	
		mcSetPrefValueInt("audioenc.vorbis.bitrate", bitrate);
		mcSetPrefValueInt("audioenc.vorbis.Mode", 1);
	} else if (!strcmp(fmt, "wma")) {
		mcSetPrefValueInt("overall.audio.encoder", AC_WMA);	
		mcSetPrefValueInt("audioenc.wma.bitrate", bitrate);
		mcSetPrefValueInt("audioenc.wma.mode", 1);
	} else if (!strcmp(fmt, "aac")) {
		mcSetPrefValueInt("overall.audio.encoder", AC_FAAC);	
		mcSetPrefValueInt("audioenc.faac.bitrate", bitrate);
		mcSetPrefValueInt("audioenc.faac.mode", 1);
	} else if (!strcmp(fmt, "aacplus")) {
		mcSetPrefValueInt("overall.audio.encoder", AC_AACPLUS);	
		mcSetPrefValueInt("audioenc.aacplus.bitrate", bitrate);
		mcSetPrefValueInt("audioenc.aacplus.mode", 1);
	} else if (!strcmp(fmt, "wav")) {
		mcSetPrefValueInt("overall.audio.encoder", AC_PCM);	
	} else {
		printf("\nInvalid target format.\n");
		goto exit;
	}
	mcApplyParam();

	// show settings
	printf("Output folder: %s\n", mcGetPrefValueStr("overall.task.destdir"));
	printf("Target format: %s\n", mcGetToolName(TL_AUDIO_ENCODER, 0, mcGetPrefValueInt("overall.audio.encoder")));
	printf("Target bit rate: %d Kbps\n", bitrate);
	i = mcGetPrefValueInt("overall.video.container");
	if (i) {
		printf("/%s", mcGetToolName(TL_CONTAINER, 0, i));
	}

	// start transcoding thread
	hThread = CreateThread(0,0,(LPTHREAD_START_ROUTINE)mcTranscodeQueue,0,0,(LPDWORD)&i);

	// show progress and wait till the transcoding finishes
	do {
		ShowStats();
	} while (WaitForSingleObject(hThread, 1000) == WAIT_TIMEOUT);

	if (mcstate->state == STATE_SHUTDOWN) {
		ShutDownNT(1);
	}

exit:
	mcUninit();
	return 0;
}
