/*******************************************************************
* MediaCoder - The universal audio/video transcoder
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*******************************************************************/

#ifndef AUDIOCODEC_H
#define AUDIOCODEC_H

// audio encoder flags
#define FLAG_RAW_PCM 0x1
#define FLAG_STANDALONE 0x4
#define FLAG_READ_CONSOLE 0x8
#define FLAG_PIPE 0x10
#define FLAG_TO_TAG 0x20
#define FLAG_NO_ENCODER 0x40
#define FLAG_QUIET 0x80

#include "enc_if.h"
#include "MACDll.h"

class CDllHelper;

class CAudioEncoder : public CEncoder
{
public:
	/* encoder methods ***********************************************************/
	virtual int Open(CString& cmd, CString& destfile, T_ITEM* item);
	virtual int Encode(char *data, int bytes);
	virtual int Close();
	virtual int GetProgress();
	int flags;
	/* encoder properties ********************************************************/
	virtual int GetLowestSampleRate() { return 8000; }
	virtual int GetHighestSampleRate() { return 48000; }
	virtual int GetMinChannel() { return 1; }
	virtual int GetMaxChannel() { return 2; }
	int framesize;
	/*****************************************************************************/
	CAudioEncoder():flags(0),percent(0),framesize(0) { prefNodePath = "audioenc"; }
	CString cl;
protected:
	CLogger logger;
	SHELL_PARAM sp;
	int percent;
};

typedef AudioCoder* (*CREATEAUDIO3TYPE)(int nch, int srate, int bps, unsigned int srct, unsigned int *outt, char *configfile);
typedef unsigned int (*GETAUDIOTYPES3TYPE)(int idx, char *desc);

class CWinampEnc
{
public:
	int open(CString& encplugfile, CString& destfile, CString& conffile, T_ITEM* item);
	int encode(char* data, int bytes);
	int close();
	CWinampEnc():encoder(0),outtype(1346584897) { logger.SetModuleName("WAENC"); }
	unsigned int outtype;
private:
	CLogger logger;
	HINSTANCE hEncPlugin;
	CREATEAUDIO3TYPE CreateAudio3;
	GETAUDIOTYPES3TYPE GetAudioTypes3;
	AudioCoder *(*finishAudio3)(char *fn, AudioCoder *c);
	void (*PrepareToFinish)(const char *filename, AudioCoder *coder);
	AudioCoder* encoder;
	FILE* fp;
	CString outfile;
};

class CLameMP3:public CAudioEncoder
{
public:
	CLameMP3();
	char* GetName() {return "Lame";}
	char* GetInExts() {return "wav";}
	char* GetFullName() { return "Lame MP3"; }
	char* GetOutExt(T_ITEM* item) {return "mp3";}
	char* GetVersionInfo();
	int GetTargetBitRate(T_ITEM* item = 0);
	int UpdateSettings(T_ITEM* item);
};

class CAACPlus:public CAudioEncoder
{
public:
	CAACPlus();
	char* GetName() {return "CT AAC+";}
	char* GetFullName() { return "CT AAC+ V2"; }
	char* GetOutExt(T_ITEM* item) {return "aac";}
	int Open(CString& cmd, CString& destfile, T_ITEM* item);
	int Encode(char *buffer, int bytes);
	int Close();
	int GetLowestSampleRate() { return 32000; }
	int GetTargetBitRate(T_ITEM* item = 0);
private:
	CWinampEnc waenc;
};

typedef int (*pfnAacPlusEncode)(char *data, int bytes);
typedef int (*pfnAacPlusInit)(char *outfile, int bitrate, int useParametric, FORMAT_CHUNK *fmt);
typedef int (*pfnAacPlusFinish)();

#define MODE_AUTO 0
#define MODE_STEREO 1
#define MODE_STEREO_NOPARAM 2
#define MODE_MONO 3

class C3gppAacPlus:public CAudioEncoder
{
public:
	C3gppAacPlus();
	char* GetName() {return "3GPP AAC+";}
	char* GetFullName() {return "3GPP AAC+ V2";}
	char* GetOutExt(T_ITEM* item) {return "m4a";}
	int Open(CString& cmd, CString& destfile, T_ITEM* item);
	int Encode(char *buffer, int bytes);
	int Close();
	int UpdateSettings(T_ITEM* item);
	int GetLowestSampleRate() { return 32000; }
	int GetTargetBitRate(T_ITEM* item = 0);
private:
	CDllHelper* dll;
	pfnAacPlusEncode AacPlusEncode;
	pfnAacPlusFinish AacPlusFinish;
};

class CFaac:public CAudioEncoder
{
public:
	CFaac();
	char* GetName() {return "FAAC";}
	char* GetOutExt(T_ITEM* item) {return "m4a";}
	char* GetVersionInfo();
	int GetTargetBitRate(T_ITEM* item = 0);
	int UpdateSettings(T_ITEM* item);
};

class CHelix:public CAudioEncoder
{
public:
	CHelix();
	char* GetName() {return "Helix";}
	char* GetFullName() { return "Helix Producer"; }
	char* GetOutExt(T_ITEM* item);
	char* GetInExts() {return "wav";}
	char* GetVersionInfo();
	int UpdateSettings(T_ITEM* item);
	int GetProgress();
};

class CAmr:public CAudioEncoder
{
public:
	CAmr();
	char* GetName() {return "AMR";}
	char* GetFullName() { return "AMR Encoder"; }
	char* GetOutExt(T_ITEM* item) {return "amr";}
	int Open(CString& cmd, CString& destfile, T_ITEM* item);
	int Encode(char *buffer, int bytes);
	int Close();
	int UpdateSettings(T_ITEM* item);
	int GetMaxChannel() { return 1; }
	int GetLowestSampleRate();
	int GetHighestSampleRate();
private:
	CDllHelper* dll;
	void* pfnEncode;
	FILE* fp;
	int codec;
    void *state;
    WORD mode;
    WORD allow_dtx;
};

class CWma:public CAudioEncoder
{
public:
	CWma();
	char* GetName() {return "WMA";}
	char* GetFullName() { return "Windows Media Audio"; }
	char* GetOutExt(T_ITEM* item) {return "wma";}
	char* GetInExts() {return "mp3;wav;mpg;avi";}
	int GetTargetBitRate(T_ITEM* item = 0);
	int UpdateSettings(T_ITEM* item);
};

class CGeneric:public CAudioEncoder
{
public:
	CGeneric();
	char* GetName() {return "CLI";}
	char* GetFullName() { return "Generic CLI Encoder"; }
	char* GetOutExt(T_ITEM* item);
	int UpdateSettings(T_ITEM* item);
};

class CWavPack:public CAudioEncoder
{
public:
	CWavPack();
	char* GetName() {return "WavPack";}
	char* GetOutExt(T_ITEM* item) {return "wv";}
	char* GetVersionInfo();
	int UpdateSettings(T_ITEM* item);
};

class CFlac:public CAudioEncoder
{
public:
	CFlac();
	char* GetName() {return "FLAC";}
	char* GetFullName() {return "Free Lossless Audio Codec";}
	char* GetOutExt(T_ITEM* item) {return "flac";}
	char* GetVersionInfo();
	int UpdateSettings(T_ITEM* item);
};

////////////////////////////////////////////////////////////////////////////////////
// Monkey's Audio Encoder 
////////////////////////////////////////////////////////////////////////////////////

// MAC_DLL structure (holds function pointers)
typedef struct {
	// APECompress functions
	proc_APECompress_Create			Create;
	proc_APECompress_Destroy		Destroy;
	proc_APECompress_Start			Start;
	proc_APECompress_AddData		AddData;
	proc_APECompress_Finish			Finish;
} MAC_COMPRESS_DLL;

class CApe:public CAudioEncoder
{
public:
	CApe();
	int Open(CString& cmd, CString& destfile, T_ITEM* item);
	int Encode(char *data, int bytes);
	int Close();
	char* GetName() {return "APE";}
	char* GetFullName() { return "Monkey's Audio"; }
	char* GetOutExt(T_ITEM* item) {return "ape";}
private:
	int GetFunctions(HMODULE hMACDll);
	int VersionCheckInterface(HMODULE hMACDll);
	HMODULE hMACDll;
	MAC_COMPRESS_DLL MACDll;
	APE_COMPRESS_HANDLE hAPECompress;
};

class CVorbis:public CAudioEncoder
{
public:
	CVorbis();
	char* GetName() {return "Vorbis";}
	char* GetFullName() { return "Ogg/Vorbis"; }
	char* GetOutExt(T_ITEM* item) {return "ogg";}
	char* GetVersionInfo();
	int GetTargetBitRate(T_ITEM* item = 0);
	int UpdateSettings(T_ITEM* item);
};

class CPcm:public CAudioEncoder
{
public:
	CPcm();
	int Open(CString& cmd, CString& destfile, T_ITEM* item);
	int Close();
	int Encode(char *buffer, int bytes);
	char* GetName() {return "PCM";}
	char* GetFullName() { return "Waveform/PCM"; }
	char* GetOutExt(T_ITEM* item);
private:
	HANDLE hFile;
	RIFF_HEADER hdr;
};

class CMusePack:public CAudioEncoder
{
public:
	CMusePack();
	char* GetName() {return "MusePack";}
	char* GetOutExt(T_ITEM* item) {return "mpc";}
	char* GetVersionInfo();
	int UpdateSettings(T_ITEM* item);
};

class COptimFrog:public CAudioEncoder
{
public:
	COptimFrog();
	char* GetName() {return "OptimFrog";}
	char* GetInExts() {return "wav";}
	char* GetOutExt(T_ITEM* item) {return "ofr";}
	char* GetVersionInfo();
	int UpdateSettings(T_ITEM* item);
};

class CMp4Als:public CAudioEncoder
{
public:
	CMp4Als();
	char* GetName() {return "AAC Lossless";}
	char* GetInExts() {return "wav";}
	char* GetOutExt(T_ITEM* item) {return "als";}
	char* GetVersionInfo();
	int UpdateSettings(T_ITEM* item);
};

class CItunesEnc:public CAudioEncoder
{
public:
	CItunesEnc();
	char* GetName() {return "iTunesEncode";}
	char* GetInExts() {return "wav;mp3;m4a;wma;aiff";}
	char* GetOutExt(T_ITEM* item) {return "m4a";}
	int UpdateSettings(T_ITEM* item);
};

class CHelixMP3:public CAudioEncoder
{
public:
	CHelixMP3();
	char* GetName() {return "HelixMP3";}
	char* GetFullName() {return "Helix MP3 Encoder";}
	char* GetInExts() {return "wav";}
	char* GetOutExt(T_ITEM* item) {return "mp3";}
	char* GetVersionInfo();
	int GetTargetBitRate(T_ITEM* item = 0);
	int UpdateSettings(T_ITEM* item);
};

class CNeroAAC:public CAudioEncoder
{
public:
	CNeroAAC();
	int Open(CString& cmd, CString& destfile, T_ITEM* item);
	int Encode(char *data, int bytes);
	int Close();
	char* GetName() {return "Nero AAC";}
	char* GetFullName() {return "Nero AAC Encoder";}
	char* GetOutExt(T_ITEM* item) {return "m4a";}
	int GetDLLDir(char* buf, int bufsize);
private:
	int SetOptions();
	HMODULE hAacEncDLL;
	HMODULE hAacDLL;
};

class CNeroRefEnc:public CAudioEncoder
{
public:
	CNeroRefEnc();
	char* GetName() {return "Nero AAC";}
	char* GetFullName() {return "Nero AAC Encoder";}
	char* GetOutExt(T_ITEM* item) {return "m4a";}
	int UpdateSettings(T_ITEM* item);
	char* GetVersionInfo();
	int GetTargetBitRate(T_ITEM* item = 0);
};

class CFFMpegAudio:public CAudioEncoder
{
public:
	CFFMpegAudio();
	char* GetName() {return "FFMPEG";}
	char* GetFullName() { return "FFMPEG Audio Codecs"; }
	char* GetOutExt(T_ITEM* item);
	char* GetVersionInfo();
	int GetTargetBitRate(T_ITEM* item = 0);
	int UpdateSettings(T_ITEM* item);
	char* GetCodecName();
};

class CAudioCopy:public CAudioEncoder
{
public:
	char* GetName() {return "Stream Copy";}
};

class CAudioNone:public CAudioEncoder
{
public:
	char* GetName() {return "No Audio";}
};

class CNullAudioEncoder:public CAudioEncoder
{
public:
	char* GetName() {return "Null";}
	char* GetFullName() { return "Null Audio Encoder"; }
	char* GetOutExt(T_ITEM* item) {return "null";}
	int open(CString& cmd, CString& destfile, T_ITEM* item)
	{
		FILE *fp = fopen(destfile.GetBuffer(), "w");
		if (!fp) return -1;
		fclose(fp);
		return 0;
	}
	int Encode(char *data, int bytes) { return bytes; }
	int close() { return 0; }
};

#endif
