/*******************************************************************
* MediaCoder - The universal audio/video transcoder
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*******************************************************************/

#ifndef AUDIOSOURCE_H
#define AUDIOSOURCE_H

#define AUDIO_SRC_COUNT 4

#define MS2BYTE(MS, BPS) (((unsigned long long)MS * BPS / 1000) & (-8))	
#define BYTE2MS(BYTES, BPS) (int)(BYTES * 100 / (BPS / 10))

enum {
	AS_MPLAYER = 0,
	AS_WINAMP,
	AS_LAME,
	AS_WAVE,
	AS_COUNT,
	AS_AMR,
};

class CAudioSource
{
public:
	virtual int open(T_ITEM* item, int startpos) = 0;
	virtual int extract(T_ITEM* item, char* outfile, int startpos) { return -1; }
	virtual int close() = 0;
	virtual int seek(int startpos) {return -1;}
	virtual int read(char* buffer, int bufsize);
	virtual int stop() {return -1;}
	virtual int GetPos();
	virtual char* GetName() = 0;
	virtual int prepare();
	CAudioSource():hInput(0),state(0),prefNodePath(0),duration(0),framesize(0),samplerate(0),item(0),bytesPerSec(44100 * 4) {}
	int state;
	unsigned long duration;
	unsigned long long bytescount;
	unsigned long long totalbytes;
	int samplerate;
	int samplebits;
	int channels;
	int framesize;
	char* prefNodePath;
	DWORD bytesPerSec;
	T_ITEM* item;
	HANDLE hInput;
protected:
	CLogger logger;
};

class CWaveFileSource:public CAudioSource
{
public:
	CWaveFileSource();
	int open(T_ITEM* item, int startpos);
	int close();
	int seek(int startpos);
	char* GetName() { return "Waveform/PCM File"; }
};

class CLameDecoder:public CAudioSource
{
public:
	CLameDecoder();
	int open(T_ITEM* item, int startpos);
	int close();
	int seek(int startpos);
	char* GetName() { return "Lame MP3 Decoder"; }
private:
	SHELL_PARAM sp;
};

class CAmrDecoder:public CAudioSource
{
public:
	CAmrDecoder();
	int open(T_ITEM* item, int startpos);
	int close();
	int read(char* buffer, int bytes);
	char* GetName() { return "AMR Decoder"; }
	int frames;
private:
	CDllHelper* dll;
	void* pfnDecode;
	FILE *fp;
	void * destate;
};

#define MP_DLL "mplayer.dll"
#define MP_EXE "mplayer.exe"

class CMplayerSource:public CAudioSource
{
public:
	CMplayerSource():sp(0),dll(0) { 
		prefNodePath = "audiosrc.mplayer"; 
		logger.SetModuleName(GetName());
	}
	//int read(char* buffer, int bufsize);
	int extract(T_ITEM* item, char* outfile, int startpos);
	int open(T_ITEM* item, int startpos);
	int close();
	char* GetName() { return "MPlayer"; }
private:
	CDllHelper* dll;
	SHELL_PARAM* sp;
};

class CWinampInput:public CAudioSource
{
public:
	CWinampInput():mod(0) {
		prefNodePath = "audiosrc.winamp"; 
		logger.SetModuleName(GetName());
	}
	int extract(T_ITEM* item, char* outfile, int startpos);
	int open(T_ITEM* item, int startpos);
	int close();
	int seek(int startpos);
	int stop();
	char* GetName() { return "Winamp Input Plugins"; }
	HANDLE hWaveFile;
	char* samplebuffer;
	int bufsize;
	int bufoffset;
	int bufreq;
	HANDLE hEventBufferReady;
private:
	In_Module* OpenInputPluginByExt(char * ext);
	In_Module* mod;
};

/*
class CArray
{
public:
	void* get(int i)
	{
		return i<count?data[i]:0;
	}
	void set(int i, void* val)
	{
		if (i>=count) {
			count+=16;
			data=(void**)realloc(data,count*sizeof(void*));
		}
		data[i]=val;
	}
	void purge()
	{
		if (data) {
			free(data);
			data=0;
			count=0;
		}
	}
	CArray():data(0),count(0) {}
	~CArray() { purge(); }
private:
	void** data;
	int count;
};

*/

#endif
