/*************************************************************************
* MediaCoder - The universal audio/video transcoder
*
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*************************************************************************/

#include "config.h"
#include <stdio.h>
#include "atlstr.h"
#include "resource.h"
#include "windowhandle.h"
#include "datatypes.h"
#include "common.h"
#include "winamphelper.h"
#include "mcdialogs.h"
#include "preference.h"

static COL_INFO cols[]={{"", 21}, {"Plugin", 90}, {"Description", 190}, {0}};
extern INPLUGIN_INFO inplugins;
extern DSPPLUGIN_INFO* dspplugins;

#ifdef _USRDLL
extern HINSTANCE hDllInst;
#endif

int CInPluginConfig::Config(HWND hwndParent)
{
	HINSTANCE hInst = 0;
#ifdef _USRDLL
	hInst = hDllInst;
#endif
	return ShowDlg(hInst, IDD_WINAMPCFG, hwndParent, 0);

}

bool CInPluginConfig::onCommand(int dlgid, int code)
{
	switch (code) {
	case BN_CLICKED:
		switch (dlgid) {
		case IDOK:
			onCommand(IDC_LIST, LBN_DBLCLK);
			break;
		case IDCANCEL:
			Close();
			break;
		}
		break;
	case LBN_DBLCLK: {
		INPLUGIN_INFO* cur = inplugins.next;
		int i, n = list.GetCount();
		for (i = 0; i < n && !list.GetSelection(i); cur = cur->next, i++);
		if (i < n) {
			TCHAR buf[MAX_PATH];
			if (inmod) UnloadWinampInPlugin(inmod);
			sprintf(buf,"%s%s",pref.GetValueStr("audiosrc.winamp.path"),cur->pluginfile);
			inmod = LoadWinampInPlugin(buf, 0);
			if (!inmod) break;
			((In_Module*)inmod)->Config(hwnd);
		}
		} break;
	}
	return true;
}

bool CInPluginConfig::onCreate()
{
	INPLUGIN_INFO* cur;
	int count;

	inmod = 0;
	list.hwnd = GetHandle(IDC_LIST);
	list.SetExtendedStype(LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT);
	list.AddColumns(cols);

	for (cur = inplugins.next, count = 0; cur; cur = cur->next, count++) {
		TCHAR* items[] = {"", cur->pluginfile, cur->desc, 0};
		list.AddItem(items);
		list.SetCheck(count, ISBITSET(cur->flags, FLAG_DISABLED) ? 0 : 1);
	}
	return true;
}

bool CInPluginConfig::onClose()
{
	INPLUGIN_INFO* cur;
	TCHAR *p = pref.GetValueStr("audiosrc.winamp.disabledPlugins");

	int count = 0;
	*p = 0;
	for (cur = inplugins.next, count = 0; cur; cur = cur->next, count++) {
		if (list.GetCheck(count)) {
			CLRBIT(cur->flags, FLAG_DISABLED);
		} else {
			SETBIT(cur->flags, FLAG_DISABLED);
			p += sprintf(p, "%s;", cur->pluginfile);
		}
	}
	*(p - 1) = 0;

	if (inmod) {
		inmod->Quit();
		FreeLibrary(inmod->hDllInstance);
	}
	EndDialog(hwnd, 0);
	return true;
}

CDspPluginConfig::CDspPluginConfig(DSPPLUGIN_INFO* dspplugins)
{
	plugins = dspplugins;
}

int CDspPluginConfig::Config(HWND hwndParent)
{
	HINSTANCE hInst = 0;
#ifdef _USRDLL
	hInst = hDllInst;
#endif
	return ShowDlg(hInst, IDD_WINAMPCFG, hwndParent, 0);
}

bool CDspPluginConfig::onCommand(int dlgid, int code)
{
	switch (code) {
	case BN_CLICKED:
		switch (dlgid) {
		case IDOK:
			onCommand(IDC_LIST, LBN_DBLCLK);
			break;
		case IDCANCEL:
			Close();
			break;
		}
		break;
	case LBN_DBLCLK: {
        int i, n = list.GetCount();
		for (i = 0; i < n && !list.GetSelection(i); i++);
		if (i < n)
			plugins[i].mod->Config(plugins[i].mod);
		} break;
	}
	return true;
}

bool CDspPluginConfig::onCreate()
{
	int count = 0;
	list.hwnd = GetHandle(IDC_LIST);
	list.SetExtendedStype(LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT);
	list.AddColumns(cols);

	LoadDspPlugins(plugins, 1);
	for (int i = 0; plugins[i].hDllPlugin; i++, count++) {
		TCHAR* items[] = {"", plugins[i].pluginfile, plugins[i].hdr->description, 0};
		list.AddItem(items);
		list.SetCheck(count, ISBITSET(plugins[i].flags, FLAG_DISABLED) ? 0 : 1);
	}
	return true;
}

bool CDspPluginConfig::onClose()
{
	TCHAR *p = pref.GetValueStr("audiosrc.winamp.enabledDspPlugins");
	int count = 0;
	*p = 0;
	for (int i = 0; plugins[i].hDllPlugin; i++, count++) {
		if (list.GetCheck(count)) {
			CLRBIT(plugins[i].flags, FLAG_DISABLED);
			p += sprintf(p, "%s;", plugins[i].pluginfile);
		} else {
			SETBIT(plugins[i].flags, FLAG_DISABLED);
		}
	}
	*(p - 1) = 0;
	UnloadDspPlugins(plugins, 0);
	EndDialog(hwnd, 0);
	return true;
}

