/*******************************************************************
* Parameter List Implementation
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*******************************************************************/

#include <windows.h>
#include <commctrl.h>
#include <shlobj.h>
#include <stdio.h>
#include "atlstr.h"
#include "resource.h"
#include "windowhandle.h"
#include "paramlist.h"
#ifdef PARAM_XML
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <iconv.h>
#include "xmlaccessor.h"
#endif
#include "preference.h"

DWORD hex2dw(TCHAR *p)
{
	register TCHAR c;
	register DWORD i=0;

	if (!p) return 0;
	while (*p && (*p==' ' || *p=='\t')) p++;
	for(c=*p;;){
		if (c>='A' && c<='F')
			c-=7;
		else if (c>='a' && c<='f')
			c-=39;
		else if (c<'0' || c>'9')
			break;
		i=(i<<4)|(c&0xF);
		c=*(++p);
	}
	return i;
}

#if 0

bool CParamList::onScroll(int dlgid, int code)
{
	TCHAR buf[16];
	int pos=SendDlgMsg(dlgid,TBM_GETPOS,0,0);
	sprintf(buf,"%d",pos);
	SendDlgMsg(IDC_EDIT,WM_SETTEXT,0,(LPARAM)buf);
	return true;
}

bool CParamList::onCommand(int dlgid, int code)
{
	int ret;
	TCHAR *p;

	switch (code){
	case BN_CLICKED:
		switch (dlgid) {
		case IDC_REVERT:
			revert();
			showValue();
			break;
		case IDC_APPLY:
			saveValue();
			break;
		case IDOK:
			saveValue();
			Close();
			break;
		case IDC_BROWSE:
			switch (params[idx].type) {
			case PT_DIR:
			case PT_FILE:
				p=(TCHAR*)malloc(params[idx].max+1);
				strcpy(p,(TCHAR*)params[idx].value);
				ret=(params[idx].type == PT_DIR) ? browseForPath(p) : browseForFile(p);
				if (ret) SendDlgMsg(IDC_EDIT,WM_SETTEXT,0,(LPARAM)p);
				free(p);
				break;
			case PT_ENUM:
			case PT_BOOL:
				SendDlgMsg(IDC_VALUE,CB_SHOWDROPDOWN,true,0);
				break;
			}
			break;
		}
		break;
	}
	return false;
}

bool CParamList::onNotify(LPNMHDR pnmh)
{
	switch (pnmh->code) {
	case TCN_SELCHANGE:
		if (pnmh->hwndFrom == GetHandle(IDC_TAB)) {
			iTab=TabCtrl_GetCurSel(GetDlgItem(hwnd,IDC_TAB));
			listVariables(-1);
		}
		break;
	case TCN_SELCHANGING:
	case TVN_SELCHANGING:
		saveValue();
		break;
	case TVN_SELCHANGED:
		if (pnmh->hwndFrom == GetHandle(IDC_TREE)) {
			idx=getIndex();
			showValue();
		}
		break;
	}
	return true;
}

void CParamList::revert()
{
	bool fSlider=false;
	switch (params[idx].type) {
	case PT_FLOAT:
		*(float*)params[idx].value = params[idx].valueDefault ? *(float*)params[idx].valueDefault : 0;
		break;
	case PT_INT:
		fSlider=(params[idx].max!=0);
	case PT_BOOL:
	case PT_PTR:
	case PT_ENUM:
		if (params[idx].value!=params[idx].valueDefault) {
			params[idx].value=params[idx].valueDefault;
		}
		if (fSlider) {
			if (params[idx].type==PT_FLOAT && params[idx].valueDefault) {
				SendDlgMsg(IDC_SLIDER,TBM_SETPOS,true,(LPARAM)(*(float*)params[idx].value = *(float*)params[idx].valueDefault));
			} else {
				SendDlgMsg(IDC_SLIDER,TBM_SETPOS,true,(LPARAM)params[idx].value);
			}			
		}
		break;
	case PT_FILE:
	case PT_DIR:
	case PT_STRING:
		if (strcmp((const TCHAR*)params[idx].value,(const TCHAR*)params[idx].valueDefault)) {
			strcpy((TCHAR*)params[idx].value,(const TCHAR*)params[idx].valueDefault);
		}
		break;
	}
	if (notify) (*notify)(idx,params[idx].value);
}

void CParamList::saveValue()
{
	HWND hValue=GetDlgItem(hwnd,IDC_VALUE);
	int ret;
	void* valuedef=params[idx].valueDefault;
	void* value=params[idx].value;
	TCHAR buf[256];

	//if (params[idx].flags & PF_READONLY) return;
	if (params[idx].type == PT_ENUM || params[idx].type == PT_BOOL) {
		hValue=GetDlgItem(hwnd,IDC_VALUE);
	} else {
		hValue=GetDlgItem(hwnd,IDC_EDIT);
	}
	switch (params[idx].type) {
	case PT_BOOL:
	case PT_ENUM:
		ret=SendMessage(hValue,CB_GETCURSEL,0,0);
		if (ret==-1) return;
		value=(void*)ret;
		break;
	case PT_FLOAT:
		{
		SendMessage(hValue,WM_GETTEXT,15,(LPARAM)buf);
		float fVal=(float)atof(buf);
		if ((params[idx].max && fVal>(float)params[idx].max) ||
			(params[idx].min && fVal<(float)params[idx].min)) {
			sprintf(buf,"The input value of %s is out of the valid range",params[idx].text);
			MsgBox(buf);
			showValue();
			break;
		}
		*(float*)value = fVal;
		}
		break;
	case PT_INT:
        SendMessage(hValue,WM_GETTEXT,15,(LPARAM)buf);
		ret=atoi(buf);
		if ((params[idx].min && ret<params[idx].min) ||
			(params[idx].max && ret>params[idx].max)) {
			sprintf(buf,"The input value of %s is out of the valid range",params[idx].text);
			MsgBox(buf);
			showValue();
			break;
		}
		value=(void*)atoi(buf);
		break;
	case PT_PTR:
        SendMessage(hValue,WM_GETTEXT,15,(LPARAM)buf);
		value=(void*)hex2dw(buf+2);
		break;
	case PT_FILE:
	case PT_DIR:
	case PT_STRING:
		value=malloc(params[idx].max+1);
		ret=SendMessage(hValue,WM_GETTEXT,params[idx].max,(LPARAM)value);
		if (ret<params[idx].min) {
			MsgBox("Not enough characters");
			showValue();
			break;
		}
		if (!strcmp((TCHAR*)value,(TCHAR*)params[idx].value)) {
			free(value);
			return;
		}
		strcpy((TCHAR*)params[idx].value,(TCHAR*)value);
		free(value);
		break;
	default:
		return;
	}
	if (params[idx].type != PT_STRING && params[idx].type != PT_FILE && params[idx].type != PT_DIR) {
		if (params[idx].value==value) return;
		params[idx].value=value;
	}
	if (notify) (*notify)(idx,value);
	return;
}

void CParamList::showValue()
{
	TCHAR *p;
	HWND hValue,hCtrl;
	void *value;
	int paramtype=params[idx].type;
	bool fEnumMode=(paramtype==PT_ENUM || paramtype==PT_BOOL);
	bool fBrowse=(paramtype==PT_DIR || paramtype==PT_FILE);
	bool fSlider=((paramtype==PT_INT || paramtype==PT_FLOAT) && params[idx].max);
	int iWindowWidth;
	int iTextBoxWidth;
	int iSliderVal;

	ShowWindow(GetHandle(IDC_EDIT),SW_HIDE);
	ShowWindow(GetHandle(IDC_BROWSE),SW_HIDE);
	ShowWindow(GetHandle(IDC_VALUE),SW_HIDE);
	ShowWindow(GetHandle(IDC_SLIDER),SW_HIDE);

	GetWindowPos(0,0,&iWindowWidth,0);
	iWindowWidth -= GetSystemMetrics(SM_CXDLGFRAME) * 2;
	iTextBoxWidth=fSlider?50:(iWindowWidth-(fBrowse?63:43));

	if (fEnumMode) {
		hValue=GetHandle(IDC_VALUE);
		SendMessage(hValue,CB_RESETCONTENT,0,0);
		//EnableWindow(hValue,!(params[idx].flags & PF_READONLY));
	} else {
		hValue=GetHandle(IDC_EDIT);
		/*
		SendMessage(hValue,EM_SETREADONLY,
			(params[idx].flags & PF_READONLY)||(paramtype>=PT_SEPERATOR),0);
			*/
	}
	hCtrl=GetHandle(IDC_BROWSE);
	if (fBrowse) ShowWindow(hCtrl,SW_SHOW);

	MoveWindow(hCtrl,iTextBoxWidth+20,iBaseLine,20,21,true);
	MoveWindow(hValue,20,iBaseLine,iTextBoxWidth,21,true);
	ShowWindow(hValue,SW_SHOW);

	if (paramtype==PT_SEPERATOR || paramtype==PT_CATALOG) {
		SendMessage(hValue,WM_SETTEXT,0,0);
		UpdateWindow(hwnd);
		goto exit;
	}

	if (paramtype==PT_ENUM && params[idx].enums) {
		p=(TCHAR*)params[idx].enums;
		do {
			if (*p == '\t') p++;
			SendMessage(hValue,CB_ADDSTRING,0,(LPARAM)p);
			while (*(p++));
		} while (*p);
	}
	value=params[idx].value;
	switch (params[idx].type & 0xf) {
	case PT_ENUM:
		SendMessage(hValue,CB_SETCURSEL,(WPARAM)value,0);
		break;
	case PT_FILE:
	case PT_DIR:
	case PT_STRING:
        SendMessage(hValue,WM_SETTEXT,0,(LPARAM)value);
		break;
	case PT_FLOAT: {
		char buf[64];
		sprintf(buf,"%.3f",*(float*)value);
		iSliderVal=(int)*(float*)value;
        SendMessage(hValue,WM_SETTEXT,0,(LPARAM)buf);
		} break;
	case PT_INT: {
		TCHAR buf[16];
		iSliderVal=(int)value;
        sprintf(buf,"%d",iSliderVal);
		SendMessage(hValue,WM_SETTEXT,0,(LPARAM)buf);
		} break;
	case PT_BOOL:
		SendMessage(hValue,CB_ADDSTRING,0,(LPARAM)"No");
		SendMessage(hValue,CB_ADDSTRING,0,(LPARAM)"Yes");
		SendMessage(hValue,CB_SETCURSEL,(WPARAM)value,0);
		break;
	case PT_PTR: {
		TCHAR buf[16];
        sprintf(buf,"0x%x",(int)value);
        SendMessage(hValue,WM_SETTEXT,0,(LPARAM)buf);
		} break;
	default:
		return;
	}

	hCtrl=GetHandle(IDC_SLIDER);
	if (fSlider) {
		SendMessage(hCtrl,TBM_SETPOS,FALSE,0);
		SendMessage(hCtrl,TBM_SETRANGEMIN,0,params[idx].min);
		SendMessage(hCtrl,TBM_SETRANGEMAX,0,params[idx].max);
		SendMessage(hCtrl,TBM_SETPAGESIZE,0,
			params[idx].enums?(LPARAM)params[idx].enums:(params[idx].max-params[idx].min)/16);
		SendMessage(hCtrl,TBM_SETPOS,TRUE,iSliderVal);
		MoveWindow(hCtrl,iTextBoxWidth+20,iBaseLine-2,iWindowWidth-iTextBoxWidth-42,24,false);
		ShowWindow(hCtrl,SW_SHOW);
	}

	// show setting tips
exit:
	if (info) {
		SendDlgMsg(IDC_INFO,WM_SETTEXT,0,0);
		for (int i=0; info[i].desc; i++) {
			if (info[i].id == idx) {
				SendDlgMsg(IDC_INFO,WM_SETTEXT,0,(LPARAM)info[i].desc);
				break;
			}
		}
	}
	UpdateWindow(hwnd);
}

CParamList::~CParamList()
{
	free(pages);
}

void CParamList::refresh()
{
	HWND hwndTab;
	TCITEM tie;
	int count=0;

	hwndTab=GetDlgItem(hwnd,IDC_TAB);
	TabCtrl_DeleteAllItems(hwndTab);
	// Add a tab for each of the three child dialog boxes. 
    tie.mask = TCIF_TEXT | TCIF_IMAGE; 
    tie.iImage = -1; 

	for (int i=0;params->type;i++) {
		if ((params->type & 0xf)==PT_CATALOG) {
			tie.pszText = (LPSTR) params->text; 
			TabCtrl_InsertItem(hwndTab, i, &tie);
			count++;
		}
	}
	if (pages) free(pages);
	if (count==0) {
		tie.pszText="General";
		TabCtrl_InsertItem(hwndTab, 0, &tie);
		pages=(int*)malloc(sizeof(int));
		pages[0]=0;
		iPageCount=1;
	} else {
		pages=(int*)malloc(count*sizeof(int));
		for (int i=0; iPageCount<count; i++) {
			if ((params->type & 0xf)==PT_CATALOG)
				pages[iPageCount++]=i;
		}
	}
}

int CParamList::showVariable(int varidx)
{
	int pg, i;
	for (pg=varidx; pg>=0 && params[pg].type!=PT_CATALOG; pg--);
	for (i=0; i<iPageCount && pages[i]!=pg; i++);
	if (i==iPageCount) return -1;
	iTab=i;
	TabCtrl_SetCurSel(GetDlgItem(hwnd,IDC_TAB),iTab);
	listVariables(varidx);
	return 0;
}

int CParamList::listVariables(int selected)
{
	int i;
	TVINSERTSTRUCT tvis;

	if (!hwnd || !params) return 0;
	hTree=GetHandle(IDC_TREE);
	TreeView_DeleteAllItems(hTree);

	tvis.item.mask=TVIF_TEXT|TVIF_PARAM|TVIF_STATE;
	tvis.hParent=TVI_ROOT;
	tvis.hInsertAfter=TVI_LAST;

	for (i=pages[iTab]+1; params->type && (params->type & 0xf)!=PT_CATALOG; i++) {
		HTREEITEM hitem;
		tvis.item.pszText=params->text?params->text:"";
		tvis.item.lParam=i;
		if (selected==i) {
			tvis.item.state=TVIS_EXPANDED|TVIS_BOLD;
			tvis.item.stateMask=TVIS_EXPANDED|TVIS_BOLD;
		} else {
			tvis.item.stateMask=0;
		}
		/*
		if (params->flags & PF_NODE) {
			if (tvis.hParent!=TVI_ROOT) {
				tvis.hParent=TreeView_GetParent(hTree,tvis.hParent);
			}
			hitem=TreeView_InsertItem(hTree,(LPARAM)&tvis);
			tvis.hParent=hitem;
		} else {
			hitem=TreeView_InsertItem(hTree,(LPARAM)&tvis);
		}
		*/
	}
	idx=pages[iTab];
	showValue();
	return i;
}

int CParamList::getIndex()
{
	TVITEM item;

	item.hItem=TreeView_GetSelection(hTree);
	item.mask=TVIF_PARAM;
	if (!TreeView_GetItem(hTree,&item)) return 0;
	return item.lParam;
}

int CParamList::browseForFile(TCHAR* buffer)
{
	OPENFILENAME ofn;
	int ret;
	TCHAR curdir[MAX_PATH];

	GetCurrentDirectory(sizeof(curdir)-1,curdir);

	ZeroMemory(&ofn,sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hInstance = GetModuleHandle(NULL);
	ofn.hwndOwner = hwnd;
	ofn.lpstrFile = buffer;
	ofn.nMaxFile = params[idx].max;
	ofn.nFilterIndex = 0;
	ofn.lpstrDefExt = NULL;
	ofn.lpstrFilter = (LPCSTR)params[idx].enums;
	
	ofn.lpstrTitle = params[idx].text;
	ofn.Flags = OFN_CREATEPROMPT;
	ret=GetOpenFileName(&ofn);
	SetCurrentDirectory(curdir);
	return ret;
}

int CParamList::browseForPath(TCHAR* buffer)
{
	LPITEMIDLIST idlist;
	BROWSEINFO udtBI;

	ZeroMemory(&udtBI,sizeof(BROWSEINFO));
	udtBI.hwndOwner=0;
	udtBI.lpszTitle=params[idx].text;
	udtBI.ulFlags= BIF_RETURNONLYFSDIRS;
	idlist=SHBrowseForFolder(&udtBI);
	if (!idlist || !SHGetPathFromIDList(idlist,buffer))
		return 0;
	CoTaskMemFree(idlist);
	while(*(++buffer));
	if (*(buffer-1)!='\\') {
		*buffer='\\';
		*(buffer+1)=0;
	}
	return 1;
}

bool CParamList::onCreate()
{
	RECT rcDialog;
	RECT rc;
	GetWindowRect(hwnd, &rcDialog);
	GetWindowRect(GetHandle(IDC_EDIT), &rc);
	AdjustWindowRect(&rc, WS_OVERLAPPEDWINDOW, FALSE);
	iBaseLine = rc.top - rcDialog.top + 1;

	hTree=GetHandle(IDC_TREE);
	refresh();
	showVariable(idx);
	Show();
	return true;
}

bool CParamList::onClose()
{
	if (notify) (*notify)(-1,0);
	EndDialog(hwnd, 0);
	return true;
}

int CParamList::open(HINSTANCE hInst, PARAM_DATA *params, PARAM_NOTIFY notify, HWND hwndParent)
{
	root=params;
	this->notify=notify;
	idx = varidx;
	iTab=0;
	iPageCount=0;
	pages=0;
	return ShowDlg(hInst, IDD_PARAM, hwndParent, 0);
}

#endif