/*************************************************************************
* MediaCoder - The universal audio/video transcoder
*
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. The name of the author may not be used to endorse or promote products
*    derived from this software without specific prior permission.
*************************************************************************/

#include "config.h"
#include "atlstr.h"
#include "windowhandle.h"
#include "processpil.h"
#include "common.h"
#include "transcode.h"
#include "mplayerwrapper.h"
#include "preference.h"
#include "resource.h"

CMPlayer player;
extern T_ITEM itemdef;
static HWND hVideo = 0;
#define OVERLAYCOLOR 0x10

void PlayerCallback(CMPlayer *player)
{
	if (player->state == STATE_IDLE && hVideo) {
		SendMessage(hVideo, WM_CLOSE, 0, 0);
		hVideo = 0;
	}
}

int StartPlayer(T_ITEM* item, int flags, CString& opts)
{
	int ret;
	mcGetMediaInfo(item);
	opts.AppendFormat(" -colorkey %d", OVERLAYCOLOR);
	player.mppath = mcGetPrefValueStr("overall.mplayer.path");
	player.flags = flags;
	player.options = opts.GetBuffer();
	player.callback = PlayerCallback;
	ret = player.Play(item->filename);
	return ret;
}

int mcPlayerStart(T_ITEM* item, HWND hWndVideo, int flags, char* extraopts)
{
	CString opts;
	char* audiofile = 0;
	int i;
	CPrefBase prefbase("overall.mplayer");
	if (player.state != STATE_IDLE) return -1;
	if (flags & PF_PREVIEW) {
		if (item && item->destfile && FileExist(item->destfile)) {
			item = mcMakeItem(item->destfile);
		} else if (inst.state == STATE_ENCODING) {
			if (!mcinst.fAudioEncoding) {
				if (inst.stage != ST_PROCESSING_VIDEO || inst.currentpass < inst.totalpass) {
					opts.Format(IDS_PREVIEW_TIP);
					pfnEvent(MC_MSG_POPUP, opts.GetBuffer());
					return -1;
				}
				audiofile = mcContainerGetFile(CT_AUDIO, 0);
			}
			T_ITEM* prvitem = mcMakeItem(instinfo.destfile.GetBuffer());
			memcpy(&prvitem->target, &item->target, sizeof(TARGET_SETTINGS));
			if (!prvitem->target.video.fps) {
				prvitem->target.video.fps = item->info.video.fps;
				prvitem->target.video.fpsScale = item->info.video.fpsScale;
			}
			if (item->target.video.fps) {
				opts += " -fps ";
				GetFPSString(item->target.video.fps, item->target.video.fpsScale, opts);
			}
			item = prvitem;
		} else {
			opts.Format(IDS_PREVIEW_TIP2);
			pfnEvent(MC_MSG_POPUP, opts.GetBuffer());
			return -1;
		}
	} else {
		if (!item) return -1;
		mcinst.startpos = item->info.begin ? item->info.begin : pref.GetValueInt("overall.decoding.startTime") * 1000;
		GenCommonOptions(item, opts, 0);
	}
	if (!(flags & (PF_NOFILTER | PF_PREVIEW))) {
		GenAudioFilterOptions(item, opts, 0);
		GenVideoFilterOptions(item, opts);
	}
	if (extraopts) {
		opts += ' ';
		opts += extraopts;
	}
	if (audiofile) {
		opts.AppendFormat(" -audiofile \"%s\"", audiofile);
	}
	if (hWndVideo) {
		opts.AppendFormat(" -nokeepaspect -wid %d", hWndVideo);
	}
	switch (pref.GetValueInt("overall.mplayer.vo")) {
	case 1:
		opts += " -vo directx";
		if (pref.GetValueBool("overall.mplayer.noAccel")) {
			opts += ":noaccel";
		}
		break;
	case 2:
		opts += " -vo null";
		break;
	}

	hVideo = hWndVideo;
	i = StartPlayer(item, 0, opts);
	if (flags & PF_PREVIEW) {
		mcFreeItem(item);
	}
	return i;
}

int mcPlayerStop()
{
	if (player.state == STATE_PLAYING) {
		player.Stop();
		return 0;
	} else {
		return -1;
	}
}

int mcPlayerCommand(char* cmd)
{
	return player.Command(cmd);
}

static int cropopts[4];
static HANDLE hMutex;

void CropDetectCallback(CMPlayer *player)
{
	if (player->state == STATE_IDLE) {
		if (hVideo)	SendMessage(hVideo, WM_CLOSE, 0, 0);
		return;
	}
	char *s;
	char *p = player->console;
	p = strstr(p, "[CROP] Crop area:");
	if (!p) return;
	s = strstr(p, ").");
	if (!s) return;
	p = strstr(p + 18, "-vf crop=");
	if (!p) return;
	p += 9;
	WaitForSingleObject(hMutex, INFINITE);
	for (int i = 0; i < 4; i++) {
		cropopts[i] = atoi(p);
		if (!(p = strchr(p, ':'))) break;
		p++;
	}
	ReleaseMutex(hMutex);
}

int mcCropDetectStart(T_ITEM* item, HWND hWndVideo)
{
	CString opts;
	CString tmp;
	if (!item) return -1;
	GenCommonOptions(item, opts, 0);
	opts.AppendFormat(" -loop 0 -nokeepaspect -wid %d -vf cropdetect=%d:2",
		hWndVideo, pref.GetValueInt("videofilter.crop.detectLimit"));
	if (StartPlayer(item, MF_REDIRECT_CONSOLE | MF_AUTO_READ_CONSOLE, opts))
		return -1;
	hMutex = CreateMutex(0, 0, 0);
	hVideo = hWndVideo;
	player.callback = CropDetectCallback;
	return 0;
}

void mcCropDetectGetResult(int result[], BOOL fStop)
{
	if (result) {
		WaitForSingleObject(hMutex, INFINITE);
		memcpy(result, cropopts, sizeof(cropopts));
		ReleaseMutex(hMutex);
	}
	if (fStop) {
		mcPlayerStop();
		player.callback = 0;
		CloseHandle(hMutex);
	}
	return;
}

