/*************************************************************************
* MediaCoder - The universal audio/video transcoder
*
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "winamphelper.h"

INPLUGIN_INFO inplugins;
DSPPLUGIN_INFO* dspplugins = NULL;
static HWND hwndWAMain;
int dspcount = 0;

//uninteresting stuff
//for the input plugin
static void SAAdd(void *data, int timestamp, int csa) {}
static void VSAAdd(void *data, int timestamp) {}
static void SAVSAInit(int maxlatency_in_ms, int srate){}
static void SAVSADeInit(){}
static void SAAddPCMData(void *PCMData, int nch, int bps, int timestamp){}
static int SAGetMode() {return 0;}
static int VSAGetMode(int *specNch, int *waveNch) {return 0;}
static void VSAAddPCMData(void *PCMData, int nch, int bps, int timestamp){}
static void VSASetInfo(int nch, int srate){}
static int dsp_isactive() {return 0;}
static int dsp_dosamples(short int *samples, int ns, int bps, int nch, int srate) {return ns;}
static void SetInfo(int bitrate0, int srate0, int stereo, int synched) {}

//for the output module
static int Pause(int p) {return 0;}
static void SetVolume(int v) {}
static void SetPan(int p) {}
static void Flush(int t) {}
static int GetOutputTime() {return 0;}
static int GetWrittenTime() {return 0;}
static void out_init(){}
static void out_quit(){}
static void out_config(HWND n){}
static void out_about(HWND n){}

static int NullCanWrite()
{
	return 1024;
}

static int NullWrite(char *buf, int len) {
	return 0;
}

static int NullOpen(int srate, int nch, int bitspersamp, int bufferlenms, int prebufferms) {
	return 0;
}

static int NullIsPlaying() {
	return 0;
}

static void Close(){
}

Out_Module outmod = {
	OUT_VER,
	"",       //description
	65537,    //id
	NULL,     //hMainWindow, filled in later
	NULL,     //hDllInstance, filled in later
	out_config,
	out_about,
	out_init,
	out_quit,
	NULL,
	Close,
	NULL,
	NULL,
	NULL,
	Pause,
	SetVolume,
	SetPan,
	Flush,
	GetOutputTime,
	GetWrittenTime,
};

In_Module* LoadWinampInPlugin(char* pluginfile, HWND hwndMain)
{
	In_Module* inmod;
	WINAMPGETINMODULE2TYPE getm;
	HMODULE hmod = LoadLibrary(pluginfile);
	if(!hmod) return NULL;
	getm = (WINAMPGETINMODULE2TYPE)GetProcAddress(hmod, "winampGetInModule2");
	if (!getm) return NULL;
	inmod = getm();
	if (!inmod) {
		FreeLibrary(hmod);
		return NULL;
	}
	//fill in winamp parts of In_Module
	outmod.hDllInstance = GetModuleHandle(NULL);
	outmod.hMainWindow = hwndMain;
	outmod.Write = NullWrite;
	outmod.CanWrite = NullCanWrite;
	outmod.IsPlaying = NullIsPlaying;
	outmod.Open = NullOpen;
	inmod->hDllInstance=hmod;
	inmod->hMainWindow=hwndWAMain;
	inmod->outMod=&outmod;
	inmod->SAAdd=SAAdd;
	inmod->VSAAdd=VSAAdd;
	inmod->dsp_isactive=dsp_isactive;
	inmod->dsp_dosamples=dsp_dosamples;
	inmod->SetInfo=SetInfo;
	inmod->SAVSAInit=SAVSAInit;
	inmod->SAVSADeInit=SAVSADeInit;
	inmod->SAAddPCMData=SAAddPCMData;
	inmod->SAGetMode=SAGetMode;
	inmod->VSAAddPCMData=VSAAddPCMData;
	inmod->VSAGetMode=VSAGetMode;
	inmod->VSASetInfo=VSASetInfo;
	inmod->Init();
	return inmod;
}

void UnloadWinampInPlugin(In_Module* inmod)
{
	HMODULE hDll = inmod->hDllInstance;
	inmod->Quit();
	FreeLibrary(hDll);
}

void CleanupWinampInPlugins(INPLUGIN_INFO* hdr)
{
	INPLUGIN_INFO* plugin;
	for (plugin = hdr->next; plugin; ) {
		INPLUGIN_INFO* next = plugin->next;
		free(plugin->exts);
		free(plugin->pluginfile);
		free(plugin);
		plugin = next;
	}
}

int ScanWinampInPlugins(char* path, INPLUGIN_INFO* hdr)
{
	int i, count = 0;
	HANDLE hFind;
	WIN32_FIND_DATA f;
	char buf[MAX_PATH];
	INPLUGIN_INFO* cur = hdr;
	INPLUGIN_INFO* tail;
	INPLUGIN_INFO* prev;

	sprintf(buf,"%sin_*.dll",path);
	hFind = FindFirstFile(buf,&f);
	if (hFind == INVALID_HANDLE_VALUE) return 0;
	do {
		In_Module* inmod;
		sprintf(buf,"%s%s", path, f.cFileName);
		inmod = LoadWinampInPlugin(buf, 0);
		if (!inmod) continue;
		if (!inmod->UsesOutputPlug || !inmod->FileExtensions) {
			UnloadWinampInPlugin(inmod);
			continue;
		}
		cur->next = (INPLUGIN_INFO*)malloc(sizeof(INPLUGIN_INFO));
		cur = cur ->next;
		cur->pluginfile = strdup(f.cFileName);
		cur->exts = strdup(inmod->FileExtensions);
		cur->desc = strdup(inmod->description);
		cur->flags = 0;
		count++;
		UnloadWinampInPlugin(inmod);
	} while (FindNextFile(hFind,&f));
	FindClose(hFind);
	tail = cur;

	prev = hdr;
	cur = hdr->next;
	for (i = 0; i < count; i++, cur = cur->next) {
		if (!cur->exts[0] || !stricmp(cur->pluginfile,"in_mp3.dll")) {
			tail->next = cur;
			tail = cur;
			prev->next = cur->next;
			continue;
		} 
		if (!stricmp(cur->exts,"APE")) {
			free(cur->exts);
			cur->exts = strdup("APE;APL");
		} else if (!stricmp(cur->exts,"MP4")) {
			free(cur->exts);
			cur->exts = strdup("MP4;M4A");
		}
		prev = cur;
	}
	tail->next = 0;
	return count;
}

/*
* Winamp DSP plugin audio filter
* Copyright (c) 2005-06 Stanley Huang
*/

static HANDLE hPluginThread = NULL;
static HANDLE hEventInit;

BOOL APIENTRY WaDSPWndProc(HWND hDlg, UINT EVENT, UINT wParam, LONG lParam)
{
    switch (EVENT)
    {
        case WM_QUIT:
            PostQuitMessage(0);
            break;
        default:
            return DefWindowProc(hDlg, EVENT, wParam, lParam);
    }
    return 0;
}

static HWND CreateMainWindow()
{
    WNDCLASS wc;
    const char *szWindowClass = "wadspclass";
	HWND hwnd;

    memset(&wc, 0, sizeof(wc));
    wc.style = CS_DBLCLKS;
    wc.lpfnWndProc = (WNDPROC) WaDSPWndProc;
    wc.hInstance = GetModuleHandle(NULL);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.lpszClassName = szWindowClass;
    RegisterClass(&wc);
    hwnd = CreateWindow(szWindowClass, "mp_wadsp", WS_DISABLED, 0, 0, 0, 0, HWND_DESKTOP, NULL, wc.hInstance, NULL);
    if (!hwnd) {
        UnregisterClass(szWindowClass, wc.hInstance);
        return 0;
    }

	return hwnd;
}

/* Main thread proc */
static int DspMainProc(DSPPLUGIN_INFO dspinfo[])
{   
    HWND hwndWAMain = CreateMainWindow();
	MSG msg;

	LoadDspPlugins(dspinfo, 0);
    SetEvent(hEventInit);

    /* the almighty Windows EVENT loop */
    while(GetMessage(&msg, NULL, 0, 0))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    //UnregisterClass(szWindowClass, wc.hInstance);
    return 0;
}

static void InitDspPlugins(DSPPLUGIN_INFO dspinfo[])
{
	DWORD ret;
	hEventInit = CreateEvent(0, 0, 0, 0);
	hPluginThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) DspMainProc, (void*)dspinfo, 0, (LPDWORD) &ret);
	WaitForSingleObject(hEventInit, INFINITE);
	CloseHandle(hEventInit);
}

int LoadDspPlugin(DSPPLUGIN_INFO* dspinfo)
{
	dspinfo->mod = dspinfo->hdr->getModule(dspinfo->modidx);
	dspinfo->mod->hDllInstance = dspinfo->hDllPlugin;
	dspinfo->mod->hwndParent = hwndWAMain;
	if (dspinfo->mod->Init(dspinfo->mod)) return -1;
	dspinfo->ModifySamples = dspinfo->mod->ModifySamples;
	return 0;
}

void UnloadDspPlugin(DSPPLUGIN_INFO* dspinfo)
{
	dspinfo->ModifySamples = 0;
	dspinfo->mod->Quit(dspinfo->mod);
}

int LoadDspPlugins(DSPPLUGIN_INFO dspinfo[], int fLoadAll)
{
	int i;
	dspcount = 0;
    for (i = 0; dspinfo[i].hDllPlugin; i++) {
		if ((ISBITSET(dspinfo[i].flags, FLAG_DISABLED) && !fLoadAll)) continue;
		dspinfo[i].mod = dspinfo[i].hdr->getModule(dspinfo[i].modidx);
		dspinfo[i].mod->hDllInstance = dspinfo[i].hDllPlugin;
		dspinfo[i].mod->hwndParent = hwndWAMain;
		LoadDspPlugin(&dspinfo[i]);
		dspcount++;
    }
    return 0;
}


/* Close stuff and deallocate memory */
void UnloadDspPlugins(DSPPLUGIN_INFO dspinfo[], int fUnloadAll)
{
	int i;
	if (!dspinfo) return;
	for (i = 0; dspinfo[i].hDllPlugin; i++) {
		if ((ISBITSET(dspinfo[i].flags, FLAG_DISABLED) || fUnloadAll)) {
			UnloadDspPlugin(&dspinfo[i]);
		}
	}
	dspcount = 0;
}

int ScanDspPlugins(char* path, DSPPLUGIN_INFO** paDspInfo)
{
	int count = 0;
	HANDLE hFind;
	WIN32_FIND_DATA f;
	char buf[MAX_PATH];

	*paDspInfo = (DSPPLUGIN_INFO*)malloc(sizeof(DSPPLUGIN_INFO));
	sprintf(buf,"%sdsp_*.dll",path);
	hFind = FindFirstFile(buf,&f);
	if (hFind == INVALID_HANDLE_VALUE) {
		(*paDspInfo)->hDllPlugin = NULL;
		return 0;
	}
	do {
		HMODULE hDllPlugin;
		winampDSPHeader* hdr;
	    winampDSPHeader *(*pfnGetHeader)(void);

		sprintf(buf, "%s%s", path, f.cFileName);
		if (!(hDllPlugin = LoadLibrary(buf))) continue;
		pfnGetHeader = (void *) GetProcAddress(hDllPlugin, "winampDSPGetHeader2");
		if (!pfnGetHeader || !(hdr = (*pfnGetHeader)())) {
			FreeLibrary(hDllPlugin);
		} else {
			DSPPLUGIN_INFO* dspinfo;
			*paDspInfo = realloc(*paDspInfo, (count + 2) * sizeof(DSPPLUGIN_INFO));
			dspinfo = *paDspInfo + count;
			memset(dspinfo, 0, sizeof(DSPPLUGIN_INFO));
			dspinfo->hdr = hdr;
			dspinfo->hDllPlugin = hDllPlugin;
			dspinfo->pluginfile = strdup(f.cFileName);
			dspinfo->mod = hdr->getModule(0);
			dspinfo->mod->hDllInstance = hDllPlugin;
			dspinfo->mod->hwndParent = hwndWAMain;
			count++;
		}
	} while (FindNextFile(hFind,&f));
	FindClose(hFind);
	(*paDspInfo + count)->hDllPlugin = NULL;
	return count;
}
