#include <windows.h>
#include <stdio.h>
#include "resource.h"
#include "..\..\include\devplugin.h"

#define DEVICE_NAME "PlayStation Portable"

int notify(int event, void* arg);

static char pspdrv = 0;

static DEV_PLUGIN_HEADER plugin= {
	DEVICE_NAME,
	DEV_TYPE_GAMECONSOLE,
	notify,
	0,
	{187, 40, 480, 272},
};

extern "C" __declspec(dllexport) DEV_PLUGIN_HEADER* GetPluginHeader()
{
	return &plugin;
}

static int FindPSPDrive()
{
	static DWORD dm = 0;
	char drv;
	DWORD d = GetLogicalDrives();
	if (d == dm) return 0;
	dm = d;
	drv = 'Z';
	for (DWORD m = (1 << 25); drv > 'C'; m >>= 1, drv--) {
		if (d & m) {
			char buf[32];
			sprintf(buf, "%c:\\", drv);
			if (GetDriveType(buf) == DRIVE_REMOVABLE) {
				DWORD ret;
				strcat(buf, "PSP");
				ret = GetFileAttributes(buf);
				if (ret != INVALID_FILE_ATTRIBUTES && (ret & FILE_ATTRIBUTE_DIRECTORY)) {
					return drv;
				}
			}
		}
	}
	return -1;
}

static void CheckPSP()
{
	char buf[64];
	char* info;
	char drv = FindPSPDrive();
	if (drv < 0) {
		info = "PSP is not connected";
		pspdrv = 0;
	} else if (drv > 0) {
		sprintf(buf, "PSP is connected at %c:", drv);
		info = buf;
		pspdrv = drv;
	} else {
		return;
	}
	SendDlgItemMessage(plugin.hWnd, IDC_INFO, WM_SETTEXT, 0, (LPARAM)info);
}

static int CALLBACK DlgProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam)
{
	switch (msg) {
	case WM_TIMER:
		CheckPSP();
		break;
	default:
		return 0;
	}
	return 1;
}

static int notify(int event, void* arg)
{
	switch (event) {
	case DEV_EVENT_LOAD:
		plugin.hWnd = CreateDialog(plugin.hInst, MAKEINTRESOURCE(IDD_PANEL), plugin.hParent, DlgProc);
		plugin.hScreen = GetDlgItem(plugin.hWnd, IDC_SKIN);
		SetTimer(plugin.hWnd, 0, 1000, 0);
		return plugin.hWnd ? 0 : -1;
	case DEV_EVENT_UNLOAD:
		KillTimer(plugin.hWnd, 0);
		DestroyWindow(plugin.hWnd);
		plugin.hWnd = 0;
		break;
	case DEV_EVENT_START:
		if (pspdrv) {
			char dir[32];
			int len = sprintf(dir, "%c:\\PSP\\VIDEO", pspdrv);
			char* p = (char*)plugin.api->pfnGetPrefValue("overall.task.destdir");
			if (strncmp(p, dir, len)) {
				char buf[128];
				sprintf(buf, "PSP is connected. Do you want to directly transcode into your PSP?\n(Output folder will be set to %s)", dir);
				if (MessageBox(plugin.hWnd, buf, "Confirm", MB_YESNO|MB_ICONQUESTION) == IDYES) {
					sprintf(p, "%s\\", dir);
					CreateDirectory(buf, 0);
				}
			}
		}
		break;
	}
	return 0;
}

BOOL APIENTRY DllMain( HINSTANCE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	switch (ul_reason_for_call) {
	case DLL_PROCESS_ATTACH:
		plugin.hInst = hModule;
		plugin.hLogo = LoadBitmap(hModule,  MAKEINTRESOURCE(IDB_LOGO));
		break;
	case DLL_PROCESS_DETACH:
		DeleteObject(plugin.hLogo);
		break;
	}
    return TRUE;
}