#include "config.h"
#include <stdio.h>
#include "atlstr.h"
#include "config.h"
#include "menc.h"
#include "cropper.h"
#include "resource.h"

#define ADJUST_X 0x1
#define ADJUST_Y 0x2
#define CF_KEEP_H 0x4
#define CF_KEEP_W 0x8
#define CF_UPDATE 0x10

#define UPPER_SPACE 138
#define MIN_WIDTH 570
#define SLIDER_WIDTH 25
#define OVERLAYCOLOR 0x10
#define DEF_VIDEO_X 8
#define DEF_VIDEO_Y 10

CVideoWindow::CVideoWindow()
{
	movable = false;
}

void CVideoWindow::Open(HWND hwndParent, int windowFlags)
{
	WNDCLASS   wc;
	HBRUSH colorbrush = CreateSolidBrush(OVERLAYCOLOR);
	memset(&wc, 0, sizeof(wc));
	wc.style         =  CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	wc.hbrBackground =  colorbrush;
	wc.lpszClassName =  "MPlayerVideoWindow";
	Create((hwndParent ? WS_CHILD : WS_OVERLAPPEDWINDOW) | windowFlags, 0, hwndParent, wc);
	mx = 0;
	my = 0;
	hParent = hwndParent;
}

bool CVideoWindow::onMouseMove(DWORD pos, int vkey)
{
	if (!movable) {
		SetCursor(LoadCursor(0, MAKEINTRESOURCE(IDC_ARROW)));
		return false;
	}
	int x = LOWORD(pos);
	int y = HIWORD(pos);
	if (vkey == MK_LBUTTON) {
		if (!mx) {
			RECT rc;
			GetWindowRect(hParent, &rc);
			mx = x + rc.left;
			my = y + rc.top;
			SetCursor(LoadCursor(0, MAKEINTRESOURCE(IDC_SIZEALL)));
			return false;
		}
	} else {
		mx = 0;
		my = 0;
		SetCursor(LoadCursor(0, MAKEINTRESOURCE(IDC_HAND)));
		return false;
	}
	int wx, wy;
	GetWindowPos(&wx, &wy);
	wx += x - mx;
	wy += y - my;
	if (wx < 0) {
		wx = wx;
	}
	Move(wx, wy);
	return false;
}

bool CVideoCropper::onMove()
{
	cropper->Notify();
	return false;
}

bool CVideoCropper::onSize(int width, int height)
{
	cropper->Notify();
	return false;
}

bool CCropper::onCreate()
{
	wndVideo.cropper = this;
	wndVideo.Open(GetHandle(IDC_SUBWND), WS_THICKFRAME);
	video.cropper = this;
	video.movable = true;
	video.Open(wndVideo.hwnd);

	ComboAddStrings(IDC_CROP_MODE, "Free Cropping\0Keep Aspect Ratio\0Fit Width\0Fit Height\0Auto Detect\0", aspect ? 1 : 0);
	ComboAddStrings(IDC_SCALE_VALUE, "1:1\0 2:1\0 3:1\0 4:1\0 8:1\0 16:1\0", 0);
	ComboAddStrings(IDC_ROUND_VALUE, "1 \0 2\0 4\0 8\0 16\0", 0);
	RECT rc = {0, 0, (vw < MIN_WIDTH ? MIN_WIDTH : vw) + 40, UPPER_SPACE + vh + 80};
	AdjustWindowRect(&rc, WS_DLGFRAME , FALSE);
	Move(0, 0, rc.right - rc.left, rc.bottom - rc.top);

	round = 8;
	SetTextInt(IDC_ROUND_VALUE, round);

	CString str;
	str.Format(IDS_CROP_TIPS);
	SetText(IDC_TIPS, str.GetBuffer());

	return true;
}

float CCropper::GetAspect()
{
	char buf[16];
	char* p;
	float newaspect = 0;
	GetText(IDC_ASPECT_RATIO, buf, sizeof(buf));
	p = strchr(buf, ':');
	if (p) {
		float f = (float)atof(p + 1);
		if (f) newaspect = (float)(atof(buf) / f);
	} else {
		newaspect = (float)atof(buf);
	}
	return newaspect;
}

bool CCropper::onCommand(int dlgid, int code)
{
	int i;
	if (holdon) return false;
	switch (code) {
	case EN_CHANGE:
		switch (dlgid) {
		case IDC_ASPECT_RATIO:
			if (aspect) {
				float newaspect = GetAspect();
				if (newaspect) {
					aspect = newaspect;
					SetCropParams(w, h, CF_UPDATE);
				}
			}
			break;
		case IDC_CROP_PARAM:
			if (!holdon) {
				char buf[32];
				char *p;
				GetText(IDC_CROP_PARAM, buf, sizeof(buf));
				for (p = buf, i = 0; *p; p++) if (*p == ':') i++;
				if (i != 3) break;
				x = atoi(buf);
				x = atoi(++(p = strchr(buf, ':')));
				w = atoi(++(p = strchr(p, ':')));
				h = atoi(++(p = strchr(p, ':')));
				holdon = true;
				SetCropParams(w, h, CF_UPDATE);
				holdon = false;
			}
			break;
		}
		break;
	default:
		return false;
	}

	return true;
}

void CCropper::SetMode(CROP_MODE newmode)
{
	mode = newmode;
	SendDlgMsg(IDC_ASPECT_RATIO, EM_SETREADONLY, FALSE, 0);
	switch (mode) {
	case CROP_FREE:
		aspect = 0;
		break;
	case CROP_DETECT:
		aspect = 0;
		SendDlgMsg(IDC_ASPECT_RATIO, EM_SETREADONLY, TRUE, 0);
		break;
	case CROP_KEEP_ASPECT:
		aspect = (float)w / h;
		break;
	case CROP_FIT_WIDTH:
		aspect = GetAspect();
		SetCropParams(0, 0, CF_UPDATE);
		break;
	case CROP_FIT_HEIGHT:
		aspect = GetAspect();
		SetCropParams(0, 0, CF_UPDATE);
		break;
	}
}

bool CCropper::onSelChange(int dlgid)
{
	char* str;
	switch (dlgid) {
	case IDC_ROUND_VALUE:
		str = ComboGetString(IDC_ROUND_VALUE);
		if (!str) break;
		round = atoi(str);
		SetCropParams(w, h, CF_UPDATE);
		free(str);
		break;
	case IDC_SCALE_VALUE:
		str = ComboGetString(IDC_SCALE_VALUE);
		if (!str) break;
		scale = atoi(str);
		Adjust();
		free(str);
		break;
	case IDC_CROP_MODE:
		SetMode((CROP_MODE)ComboGetSel(IDC_CROP_MODE));
		break;
	}
	return true;
}

bool CCropper::onSize(int width, int height)
{
	MoveWindow(GetHandle(IDC_SUBWND), 5, UPPER_SPACE + 5, width - 10, height - UPPER_SPACE - 10, TRUE);
	return true;
}

void CCropper::Apply()
{
	if (!item) return;
	item->target.crop.x = x;
	item->target.crop.y = y;
	item->target.crop.w = w;
	item->target.crop.h = h;
	mcSetPrefValueInt("videofilter.crop.enabled", 1);
	RefreshOptionTab();
	RefreshGui();
}

bool CCropper::onClick(int dlgid)
{
	switch (dlgid) {
	case IDC_PLAY:
		paused = !paused;
		if (!mcPlayerCommand("pause")) break;
		break;
	case IDC_BACKWARD:
		mcPlayerCommand("seek -10");
		break;
	case IDC_FORWARD:
		mcPlayerCommand("seek 10");
		break;
	case IDOK:
		Apply();
		applied = true;
		Close();
		break;
	case IDC_CROP_DETECT:
		onTimer(0);
		break;
	}
	return true;
}

bool CCropper::onTimer(int id)
{
	if (mode == CROP_DETECT) {
		int opts[4];
		mcCropDetectGetResult(opts, FALSE);
		x = opts[2];
		y = opts[3];
		w = opts[0];
		h = opts[1];
		SetCropParams(w, h, CF_UPDATE);
	}
	return true;
}

int CCropper::Open(CWindow* wndParent, T_ITEM *item)
{
	if (hwnd) return 0;
	holdon = true;
	vw = item->info.video.width;
	vh = item->info.video.height;
	if (vw == 0 && vh == 0) return -1;
	x = item->target.crop.x;
	y = item->target.crop.y;
	w = vw;
	h = vh;
	int i;
	if (i = item->target.crop.w)
		w = i;
	if (i = item->target.crop.h)
		h = i;
	this->item = item;

	CreateDlg(0, IDD_CROP, wndParent);
	if (!hwnd) return 0;
	wndParent->Disable();

	if (aspect) {
		char buf[8];
		int cw;
		int ch;
		cw = vw;
		ch = (int)((float)vw / aspect);
		if (ch > vh) {
			ch = vh;
			cw = (int)((float)vh * aspect);
		}
		x = (vw - cw) / 2;
		y = (vh - ch) / 2;
		SetCropParams(cw, ch);
		sprintf(buf, "%.3f:1", aspect);
		SetText(IDC_ASPECT_RATIO, buf);
		SetItemChecked(IDC_LOCK_ASPECT);
	}

	if (mcCropDetectStart(item, video.hwnd)) return -1;
	Adjust();
	SetTimer(hwnd, 0, 1000, 0);
	holdon = false;
	applied = false;
	SetMode(aspect ? CROP_KEEP_ASPECT : CROP_FREE);
	Notify();
	wndVideo.Show();
	video.Show();
	return 0;
}

void CCropper::SetCropParams(int cw, int ch, int flag)
{
	if (aspect == 0) {
		w = cw;
		h = ch;
	} else if (mode == CROP_FIT_WIDTH) {
		w = vw;
		h = (int)((float)w / aspect);
		x = 0;
		y = (vh - h) / 2;
	} else if (mode == CROP_FIT_HEIGHT) {
		h = vh;
		w = (int)(aspect * h);
		x = (vw - w) / 2;
		y = 0;
	} else if (flag & CF_KEEP_H) {
		w = (int)(aspect * ch);
		h = ch;
	} else {
		h = (int)((float)cw / aspect);
		w = cw;
	}
	w = w - w % round;
	if (w < round) w = round;
	h = h - h % round;
	if (h < round) h = round;

	char buf[32];
	_snprintf(buf, sizeof(buf), "%d:%d:%d:%d", x, y, w, h);
	holdon = true;
	SendDlgMsg(IDC_CROP_PARAM, WM_SETTEXT, 0, (LPARAM)buf);
	holdon = false;

	float f;
	if (!aspect) {
		f = (float)w / h;
		sprintf(buf, "%.3f:1", f);
		SetText(IDC_ASPECT_RATIO, buf);
	}
	f = (float)w / 1000 * h / 1000;
	sprintf(buf, "%.2fM", f);
	SendDlgMsg(IDC_PIXEL_NUMBER, WM_SETTEXT, 0, (LPARAM)buf);

	if (flag & CF_UPDATE)
		Adjust();
}

void CCropper::Notify(int code)
{
	RECT rc;
	RECT rcout;
	if (holdon) return;
	
	GetWindowRect(video.hwnd, &rc);
	GetWindowRect(wndVideo.hwnd, &rcout);
	rcout.left += GetSystemMetrics(SM_CXSIZEFRAME);
	rcout.right -= GetSystemMetrics(SM_CXSIZEFRAME);
	rcout.top += GetSystemMetrics(SM_CYSIZEFRAME);
	rcout.bottom -= GetSystemMetrics(SM_CYSIZEFRAME);
	x = (rcout.left - rc.left - 1) * scale;
	y = (rcout.top - rc.top - 1) * scale;
	SetCropParams((rcout.right - rcout.left) * scale, (rcout.bottom - rcout.top) * scale, CF_UPDATE);
}

void CCropper::Adjust()
{
	holdon = true;
	wndVideo.Move(DEF_VIDEO_X, DEF_VIDEO_Y, w / scale + GetSystemMetrics(SM_CXSIZEFRAME) * 2, h / scale + GetSystemMetrics(SM_CYSIZEFRAME) * 2);
	video.Move(-(x / scale), -(y / scale), (vw / scale), (vh / scale));
	holdon = false;
}

bool CCropper::onClose()
{
	if (!applied && Ask(IDS_CROP_CONFIRM, MB_YESNO) == IDYES) {
		Apply();		
	}
	wndParent->Enable();
	Hide();
	KillTimer(hwnd, 0);
	mcCropDetectGetResult(0, TRUE);
	video.Destroy();
	wndVideo.Destroy();
	wndParent->SetForeground();
	return false;
}
