/*************************************************************************
* MediaCoder - The universal audio/video transcoder
*
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*************************************************************************/

#include "config.h"
#include "resource.h"
#include "atlstr.h"
#include "common.h"
#include "menc.h"
#include "optiondialogs.h"
#include "deviceplugins.h"
#include <math.h>

extern char apppath[];
extern MC_STATE* mcstate;
extern CMenc mc;

CDeviceDialog device;

/*
static PROFILE_INFO profiles[] = {
	{"SP Level 0", 176, 144, 15, 48},
	{"SP Level 1", 176, 144, 15, 64},
	{"SP Level 2", 352, 288, 15, 128},
	{"SP Level 3", 352, 288, 15, 384},
	{"ASP Level 0", 176, 144, 30, 64},
	{"ASP Level 1", 176, 144, 30, 128},
	{"ASP Level 2", 352, 288, 15, 384},
	{"ASP Level 3", 352, 288, 30, 768},
	{"ASP Level 4", 352, 576, 30, 3000},
	{"ASP Level 5", 720, 576, 30, 8000},
	{0},
};
*/
static int idLabel[] = {IDC_CAT1, IDC_CAT2, IDC_CAT3};
static int idCombo[] = {IDC_PRESET1, IDC_PRESET2, IDC_PRESET3};

static char* deviceName[] = {"Device", "Mobile Phone", "Digital Media Player", "Game Console", "Disc Player"};

/*
static PHONE_PRESET presets[] = {
	{"Custom", -1, -1, 30, -1, 44100},
	{"Minimo", VIDEO_MPEG4, RES_128_64|RES_128_96|RES_160_96|RES_160_120|RES_208_176|RES_320_240, 30, AUDIO_AAC|AUDIO_STEREO, 44100},
	{"Samsung E438", VIDEO_MPEG4, RES_128_64|RES_128_96, 15, AUDIO_AAC, 44100},
	{"Nokia N70", VIDEO_MPEG4, RES_176_144|RES_208_176|RES_320_240, 15, AUDIO_AAC|AUDIO_STEREO, 44100},
	{0},
};
*/

static void devEnqueueItem(void* item, char* alias)
{
	mc.AddItem((T_ITEM*)item, alias);
}

static void devRefreshGui(int mode)
{
	switch (mode) {
	case 0:
        mc.Refresh();
		RefreshAudioTab();
		RefreshOptionTab();
	case 1:
		device.Refresh();
		break;
	}
}

static void devShowSettings(char* key)
{
	ShowWeb(mcShowConfig(CD_PREF, key));
}

static DEV_PLUGIN_API api = { mcGetPrefValue, mcSetPrefValue, EnqueueFile, devEnqueueItem, devRefreshGui, devShowSettings};

CDeviceDialog::CDeviceDialog()
{
	static PREF_CTRL s_ctrls[] = {
		{IDC_START_TIME, "overall.decoding.startTime"},
		{IDC_LENGTH, "overall.decoding.duration"},
		{IDC_OUTPUT_FOLDER, "overall.task.destdir"},
		{0}
	};
	ctrls = s_ctrls;
	dialogid = IDD_CELL;
	memset(presetInfo, 0, sizeof(presetInfo));
	plugins = 0;
	index = -1;
	devtype = -1;
	hDefLogo = LoadBitmap(hInstance,MAKEINTRESOURCE(IDB_DEVICE_LOGO));
}

void CDeviceDialog::RevertTexts()
{
	texts[TEXT_DEVICE_NAME] = "Device:";
	texts[TEXT_PLAY_BUTTON] = "&Play";
	texts[TEXT_PLAYSTOP_BUTTON] = "Sto&p";
	texts[TEXT_PREVIEW_BUTTON] = "Pre&view";
	texts[TEXT_CROP_BUTTON] = "&Crop";
	texts[TEXT_START_BUTTON] = "&Start";
	texts[TEXT_STOP_BUTTON] = "Sto&p";
	texts[TEXT_FF_BUTTON] = "S&kip 10s";
	texts[TEXT_SET_TIME] = "Set &Time";
}

void CDeviceDialog::SetTexts()
{
	SetText(IDC_TEXT_DEVICE, texts[TEXT_DEVICE_NAME]);
	SetText(IDC_PLAY, texts[playing == 1 ? TEXT_PLAYSTOP_BUTTON : TEXT_PLAY_BUTTON]);
	SetText(IDC_PREVIEW, texts[playing == 2 ? TEXT_PLAYSTOP_BUTTON : TEXT_PREVIEW_BUTTON]);
	SetText(IDC_CROP, texts[TEXT_CROP_BUTTON]);
	SetText(IDC_FF, texts[TEXT_FF_BUTTON]);
	SetText(IDC_SETTIME, texts[TEXT_SET_TIME]);
	SetText(IDC_START, texts[transcoding ? TEXT_STOP_BUTTON : TEXT_START_BUTTON]);
}

/*
PROFILE_INFO* GetProfile(int w, int h, float fps, int bitrate)
{
	for (int i = 0; profiles[i].name && 
		(w > profiles[i].w ||
		h > profiles[i].h ||
		fps > profiles[i].fps ||
		bitrate > profiles[i].bitrate); i++);
	return profiles[i].name ? &profiles[i] : 0;
}
*/

bool CDeviceDialog::onCreate()
{
	return true;
}

int CDeviceDialog::LoadPlugins(int type)
{
	WIN32_FIND_DATA f;
	CString path;
	HMODULE hDll;
	int pluginCount = 0;
	DEV_PLUGIN_INFO** queue = &plugins;
	path.Format("%splugins\\dev_*.dll", apppath);
	HANDLE hFind = FindFirstFile(path.GetBuffer(), &f);
	if (hFind != INVALID_HANDLE_VALUE) {
		do {
			DEV_PLUGIN_HEADER* hdr;
			path.Format("%s\\plugins\\%s", apppath, f.cFileName);
			hDll = LoadLibrary(path.GetBuffer());
			if (!hDll) continue;
			PFN_GET_HEADER pfnGetPlugin = (PFN_GET_HEADER)GetProcAddress(hDll, "GetPluginHeader");
			if (!pfnGetPlugin || !(hdr = pfnGetPlugin()) || LOWORD(hdr->flags) != devtype) {
				FreeLibrary(hDll);
				continue;
			}
			*queue = (DEV_PLUGIN_INFO*)malloc(sizeof(DEV_PLUGIN_INFO));
			hdr->api = &api;
			(*queue)->hdr = hdr;
			(*queue)->pluginFile = strdup(f.cFileName);
			queue = &(*queue)->next;
			pluginCount++;
		} while (FindNextFile(hFind, &f));
		*queue = 0;	// end of the chain
		FindClose(hFind);
	}
	return pluginCount;
}

void CDeviceDialog::UnloadPlugins()
{
	if (index >= 0) {
		GetPluginHeader()->notify(DEV_EVENT_UNLOAD, 0);
		xml.Close();
		index = -1;
	}
	for (DEV_PLUGIN_INFO* ptr = plugins; ptr;) {
		DEV_PLUGIN_INFO* tmp;
		DEV_PLUGIN_HEADER* hdr = ptr->hdr;
		FreeLibrary(hdr->hInst);
		free(ptr->pluginFile);
		tmp = ptr;
		ptr = ptr->next;
		free(tmp);
	}
	pluginFile = 0;
	plugins = 0;
	devtype = -1;
}


int CDeviceDialog::Open(char* pluginToShow)
{
	if (!pluginToShow) return -1;
	DEV_PLUGIN_HEADER* hdr;
	int devtype;
	CString path;
	path.Format("%s\\plugins\\%s", apppath, pluginToShow);
	HMODULE hDll = LoadLibrary(path.GetBuffer());
	if (!hDll) return -1;
	PFN_GET_HEADER pfnGetPlugin = (PFN_GET_HEADER)GetProcAddress(hDll, "GetPluginHeader");
	if (!pfnGetPlugin || !(hdr = pfnGetPlugin())) {
		FreeLibrary(hDll);
		return -1;
	}
	devtype = LOWORD(hdr->flags);
	FreeLibrary(hDll);
	return Open(devtype, pluginToShow);
}

int CDeviceDialog::Open(int type, char* pluginToShow)
{

	if (devtype != type) {
		UnloadPlugins();
		devtype = type;
		if (!LoadPlugins(type)) {
			devtype = -1;
			MsgBox(IDS_NO_DEV_PLUGIN);
			return -1;
		}
	}
	if (!hwnd) {
		CreateDlg(0, dialogid, this);
		RefreshDialog();
	}

	if (type > 0 && type < sizeof(deviceName) / sizeof(char*)) {
		char buf[64];
		_snprintf(buf, sizeof(buf), "MediaCoder for %s", deviceName[devtype]);
		SetCaption(buf);
	}

	playing = false;
	transcoding = false;
	index = -1;
	ComboReset(IDC_MODEL);
	int showidx = 0;
	int i;
	DEV_PLUGIN_INFO* ptr;
	for (ptr = plugins, i = 0; ptr; ptr = ptr->next, i++) {
		DEV_PLUGIN_HEADER* hdr = ptr->hdr;
		ComboAddString(IDC_MODEL, hdr->name);
		if (pluginToShow && !strcmp(pluginToShow, ptr->pluginFile)) {
			showidx = i;
		}
	}
	ComboSetSel(IDC_MODEL, showidx);
	ShowPlugin(showidx);

	Show();
	return 0;
}

CDeviceDialog::~CDeviceDialog()
{
	Close();
	DeleteObject(hDefLogo);
	for (int i = 0; i < PRESET_CAT; i++) {
		if (presetInfo[i]) {
			free(presetInfo[i]);
			presetInfo[i] = 0;
		}
	}
}

DEV_PLUGIN_INFO* CDeviceDialog::GetPluginInfo(int idx)
{
	int i = 0;
	if (idx < 0) idx = index;
	for (DEV_PLUGIN_INFO* ptr = plugins; ptr && i < idx; ptr = ptr->next, i++);
	return ptr ? ptr : 0;
}

DEV_PLUGIN_HEADER* CDeviceDialog::GetPluginHeader(int idx)
{
	DEV_PLUGIN_INFO* ptr = GetPluginInfo(idx);
	return ptr ? ptr->hdr : 0;
}

DEV_PLUGIN_HEADER* CDeviceDialog::GetPluginHeader(char* name)
{
	int i = 0;
	for (DEV_PLUGIN_INFO* ptr = plugins; ptr && strcmp(ptr->hdr->name, name); ptr = ptr->next);
	return ptr ? ptr->hdr : 0;
}

int CDeviceDialog::NotifyPlugin(int event, void* arg)
{
	PFN_PLUGIN_NOTIFY pfnNotify = GetPluginHeader()->notify;
	return pfnNotify ? (*pfnNotify)(event, arg) : -1;
}

bool CDeviceDialog::onChildDestroy()
{
	Refresh();
	return true;
}

bool CDeviceDialog::onTimer(int id)
{
	switch (id) {
	case 0:
		if (playing) {
			if (mcPlayerCommand("get_time_length")) {
				playing = 0;
				SetTexts();
				KillTimer(hwnd, 0);
			}
		}
		break;
	case 1:
		if (mcstate->state != STATE_ENCODING) {
			transcoding = 0;
			SendDlgItemMessage(hwnd,IDC_PROGRESS,PBM_SETPOS,0,0);
			SetTexts();
			KillTimer(hwnd, 1);
		} else {
			SendDlgItemMessage(hwnd,IDC_PROGRESS,PBM_SETPOS,GetFileProgress(),0);
		}
		break;
	}
	return true;
}

bool CDeviceDialog::onClick(int dlgid)
{
	switch (dlgid) {
	case IDC_PLAY:
		if (!playing) {
			T_ITEM* item = GetSelectedItem();
			if (item) {
				mcGetMediaInfo(item);
				if (NotifyPlugin(DEV_EVENT_PLAY, item)) break;
				DEV_PLUGIN_HEADER* hdr = GetPluginHeader();
				video.Open(hdr->hScreen, WS_VISIBLE);
				if (mcPlayerStart(item, video.hwnd, 0, "-nokeepaspect")) break;
				playing = 1;
				SetTexts();
				SetTimer(hwnd, 0, 1000, 0);
				video.Move(hdr->screen.x, hdr->screen.y, hdr->screen.w, hdr->screen.h);
			} else {
				MsgBox(IDS_NO_FILE);
			}
		} else if (playing == 1) {
			if (NotifyPlugin(DEV_EVENT_STOPPLAY)) break;
			mcPlayerStop();
		}
		break;
	case IDC_PREVIEW:
		if (!playing) {
			if (NotifyPlugin(DEV_EVENT_PREVIEW)) break;
			DEV_PLUGIN_HEADER* hdr = GetPluginHeader();
			video.Open(hdr->hScreen, WS_VISIBLE);
			if (!mcPlayerStart(GetSelectedItem(), video.hwnd, PF_PREVIEW, "-nokeepaspect")) {
				playing = 2;
				SetTexts();
				SetTimer(hwnd, 0, 1000, 0);
				video.Move(hdr->screen.x, hdr->screen.y, hdr->screen.w, hdr->screen.h);
			}
		} else if (playing == 2) {
			if (NotifyPlugin(DEV_EVENT_STOPPLAY)) break;
			mcPlayerStop();
		}
		break;
	case IDC_CROP: {
		T_ITEM* item = GetSelectedItem();
		if (NotifyPlugin(DEV_EVENT_CROP, item)) break;
		float aspect = 0;
		int w = mcGetPrefValueInt("videofilter.scale.width");
		int h = mcGetPrefValueInt("videofilter.scale.height");
		if (item && !mcGetMediaInfo(item)) {
			aspect = (float)w / h;
		}
		ShowCropper(this, aspect);
		} break;
	case IDC_START:
		if (mcstate->state == STATE_ENCODING) {
			if (!NotifyPlugin(DEV_EVENT_STOP))
				mcSetState(STATE_STOPPING);
		} else if (!NotifyPlugin(DEV_EVENT_START)) {
			ApplySettings();
			SetTimer(hwnd, 1, 1000, 0);
			transcoding = 1;
			SetTexts();
			StartTranscode();
		}
		break;
	case IDC_SETTIME:
		popdlg.SetDlg(new CDlgTime);
		popdlg.Open(this);
		Refresh();
		break;
	case IDC_FF:
		mcPlayerCommand("seek 10");
		break;
	case IDC_LOGO:
		if (pluginUrl) ShellExecute(hwnd, "open", pluginUrl, 0, 0, SW_SHOW);
		break;
	case IDC_BROWSE:
		if (BrowseForPath(mcGetPrefValueStr("overall.task.destdir")))
			RefreshDialog();
		break;
	}
	return true;
}

void CDeviceDialog::AdjustPluginWindow(HWND hWndPlugin, int catcount)
{
	RECT rc, rc2;
	int pw, ph, w, h;
	HWND hInfo = GetHandle(IDC_INFO);
	Hide();
	for (int i = 0; i < PRESET_CAT; i++) {
		ShowCtrl(idLabel[i], (i < catcount));
		ShowCtrl(idCombo[i], (i < catcount));
	}
	GetWindowRect(hwnd, &rc);
	if (catcount) {
		GetWindowRect(GetHandle(idCombo[catcount - 1]), &rc2);
		ph = rc2.bottom - rc.top - GetSystemMetrics(SM_CYCAPTION) + 6;
	} else {
		GetWindowRect(GetHandle(idLabel[0]), &rc2);
		ph = rc2.top - rc.top - GetSystemMetrics(SM_CYCAPTION);
	}
	pw = 180;
	if (!hWndPlugin) {
		w = 180;
		h = ph;
		MoveWindow(hInfo, pw, 10, w - 6, h - 18, FALSE);
	} else if (!(flags & DEV_FLAG_DIALOG)) {
		GetWindowRect(hWndPlugin,&rc);
		w = rc.right - rc.left;
		h = rc.bottom - rc.top; 
		MoveWindow(hWndPlugin,pw,0,w,h,TRUE);
		MoveWindow(hInfo, 9, ph, pw - 16, h - ph - 8, FALSE);
	} else {
		GetWindowRect(hWndPlugin,&rc);
		MoveWindow(hWndPlugin, 9, ph, rc.right - rc.left, rc.bottom - rc.top, FALSE);
		w = 180;
		h = ph + rc.bottom - rc.top;
		MoveWindow(hInfo, pw, 10, w - 6, h - 18, FALSE);
	}

	GetClientRect(hwnd, &rc);
	rc.right = pw + w;
	rc.bottom = h;

	AdjustWindowRect(&rc, WS_OVERLAPPEDWINDOW, 0);
	w = rc.right - rc.left;
	h = rc.bottom - rc.top;
	GetWindowRect(mc.hwnd, &rc2);
	if (rc2.right - rc2.left + w > GetSystemMetrics(SM_CXSCREEN)) {
		flags &= ~DEV_FLAG_DOCK;
	}
	if (!(flags & DEV_FLAG_DOCK)) {
		MoveCenter(w, h);
	} else if (rc2.left < w) {
		mc.Move(w, ORIGINAL);
		MoveWindow(hwnd, 0, rc2.top, w, h, TRUE);
	} else {
		MoveWindow(hwnd, rc2.left - w, rc2.top, w, h, TRUE);
	}
	Show();
}

void CDeviceDialog::ShowPlugin(int idx)
{
	DEV_PLUGIN_INFO* info = GetPluginInfo(idx);
	if (!info) return;
	int ret = 0;
	DEV_PLUGIN_HEADER* hdr = info->hdr;
	pluginFile = info->pluginFile;

	// unload and previous plugin first
	if (index >= 0) {
		DEV_PLUGIN_HEADER* prevhdr = GetPluginHeader();
		if (prevhdr->hWnd) ShowWindow(prevhdr->hWnd, SW_HIDE);
		prevhdr->notify(DEV_EVENT_UNLOAD, 0);
		xml.Close();
	}

	for (int i = 0; i < PRESET_CAT; i++) {
		if (presetInfo[i]) {
			free(presetInfo[i]);
			presetInfo[i] = 0;
		}
		ComboReset(idCombo[i]);
		SetText(idLabel[i], 0);
	}
	RevertTexts();
	hdr->hParent = hwnd;
	hdr->notify(DEV_EVENT_LOAD, 0);
	if (hdr->hWnd) ShowWindow(hdr->hWnd, SW_SHOW);
	
	// load presets
	CString path;
	int cat = 0;
	pluginUrl = 0;
	flags = 0;

	if (!hdr->xml) {
		CString path;
		path.Format("%splugins\\%s", apppath, info->pluginFile);
		path.Replace(".dll", ".xml");
		ret = xml.Open(path.GetBuffer());
	} else {
		ret = xml.Read(hdr->xml);
	}

	if (ret) goto done;
	path.Format("/%s/%s", PLUGIN_XML_ROOT, "properties");
	if (xml.goToKey(path.GetBuffer()) && xml.goChild()) {
		do {
			if (xml.isMatched("url")) {
				pluginUrl = xml.getNodeValue();
			} else if (xml.isMatched("name")) {
				hdr->name = xml.getNodeValue();				
			} else if (xml.isMatched("flag")) {
				static char* flagname[] = {"dock", "summary", "dialog", "confirm"};
				int flagmask = 1;
				for (int i = 0; i < sizeof(flagname) / sizeof(char*); i++, flagmask <<= 1) {
					char *p = xml.getAttribute(flagname[i]);
					if (p && *p != '0')
						flags |= flagmask;
					else
						flags &= ~flagmask;
				}
			}
		} while (xml.goNext());
	}
	path.Format("/%s/%s", PLUGIN_XML_ROOT, "presets");
	if (!xml.goToKey(path.GetBuffer()) || !xml.goChild()) goto done;
	do {
		if (!xml.isMatched("catalog")) continue;
		int count = xml.getChildCount();
		if (count <= 0) continue;
		presetInfo[cat] = (DEV_PLUGIN_PRESET*)malloc(count * sizeof(DEV_PLUGIN_PRESET));
		xml.goChild();
		int i = 0;
		int sel = 0;
		do {
			if (xml.isMatched("preset")) {
				presetInfo[cat][i].name = xml.getAttribute("text");
				if (!presetInfo[cat][i].name) continue;
				presetInfo[cat][i].key = xml.getAttribute("key");
				presetInfo[cat][i].url = xml.getAttribute("url");
				ComboAddString(idCombo[cat], presetInfo[cat][i].name);
				if (xml.getAttribute("sel")) sel = i;
				i++;
			}
		} while (xml.goNext());
		xml.goParent();
		SetText(idLabel[cat], xml.getAttribute("text"));
		ComboSetSel(idCombo[cat], sel);
		cat++;
	} while (cat < PRESET_CAT && xml.goNext());

done:
	SetTexts();
	AdjustPluginWindow(hdr->hWnd, cat);
	SendDlgMsg(IDC_LOGO,BM_SETIMAGE,IMAGE_BITMAP,(LPARAM)(hdr->hLogo ? hdr->hLogo : hDefLogo));

	index = idx;
	ApplySettings();
	Refresh();
	if (ret) {
		MsgBox(IDS_PLUGIN_XML_ERROR);	
	}
}

bool CDeviceDialog::onSelChange(int dlgid)
{
	switch (dlgid) {
	case IDC_MODEL:
		mcPlayerStop();
		ShowPlugin(ComboGetSel(IDC_MODEL));
		break;
	default:
		ApplySettings();
		Refresh();
	}
	return true;
}

bool CDeviceDialog::onClose()
{
	refreshing++;
	Hide();
	mcPlayerStop();
	if (mcGetState() != STATE_SHUTDOWN && (flags & DEV_FLAG_CONFIRM) && Ask(IDS_DEVICE_TIP, MB_YESNO) == IDNO) {
		mc.RevertPref();
	}
	UnloadPlugins();
	refreshing--;
	return false;
}

void CDeviceDialog::Refresh()
{
	HWND hTree=GetHandle(IDC_INFO);
	if (NotifyPlugin(DEV_EVENT_REFRESH) || !(flags & DEV_FLAG_SUMMARY)) return;

	CString text;
	int i;
	int vbitrate = 0;
	TVINSERTSTRUCT tvis;
	char buf[256];
	char *p;
	ShowWindow(hTree, SW_HIDE);
	TreeView_DeleteAllItems(hTree);
	memset(&tvis, 0, sizeof(tvis));
	tvis.item.mask=TVIF_TEXT|TVIF_STATE;
	tvis.hParent = TVI_ROOT;
	tvis.hInsertAfter = TVI_LAST;
	tvis.item.pszText = buf;
	tvis.item.state = TVIS_EXPANDED;
	tvis.item.stateMask = TVIS_EXPANDED | TVIS_BOLD;
	tvis.hParent = TVI_ROOT;

	int audio = mcGetPrefValueInt("overall.audio.encoder");
	int video = mcGetPrefValueInt("overall.video.format");
	int container = mcGetPrefValueInt("overall.video.container");

	if (video != VC_NONE) {
		strcpy(buf, "Video");
		tvis.item.state |= TVIS_BOLD;
		tvis.hParent = TreeView_InsertItem(hTree,(LPARAM)&tvis);
		tvis.item.state &= ~TVIS_BOLD;
		switch (video) {
		case VC_XVID:
			p = "MPEG-4";
			break;
		case VC_H264:
			p = "MPEG-4 AVC";
			break;
		case VC_MPEG1:
			p = "MPEG-1";
			break;
		case VC_MPEG2:
			p = "MPEG-2";
			break;
		default:
			p = mcGetToolName(TL_VIDEO_CODEC, 0, video);
		}
		if (p) {
			sprintf(buf, "Format: %s", p);
			TreeView_InsertItem(hTree,(LPARAM)&tvis);
		}
		if (mcGetPrefValueInt("overall.video.mode") == 1) {
			_snprintf(buf, sizeof(buf), "Quality: %d%%", mcGetPrefValueInt("overall.video.quality"));
		} else {
			vbitrate = mcGetPrefValueInt("overall.video.bitrate");
			_snprintf(buf, sizeof(buf), "Bit Rate: %d Kbps", vbitrate);
		}
		TreeView_InsertItem(hTree,(LPARAM)&tvis);
		if (mcGetPrefValueInt("videofilter.scale.enabled")) {
			_snprintf(buf, sizeof(buf), "Resolution: %dx%d",
				mcGetPrefValueInt("videofilter.scale.width"),
				mcGetPrefValueInt("videofilter.scale.height"));
			TreeView_InsertItem(hTree,(LPARAM)&tvis);
		}
		float fps = (float)GetFPS(mcGetPrefValueInt("overall.video.fps"), mcGetPrefValueInt("overall.video.fpsScale"));
		if (fps) {
			sprintf(buf, "Frame Rate: %.2f fps", fps);
			TreeView_InsertItem(hTree,(LPARAM)&tvis);
		}
		switch (mcGetPrefValueInt("videofilter.rotate.mode")) {
		case 2:
			p = "CW";
			break;
		case 3:
			p = "CCW";
			break;
		default:
			p = 0;
		}
		if (p) {
			sprintf(buf, "Rotation: %s", p);
			TreeView_InsertItem(hTree,(LPARAM)&tvis);
		}

		tvis.hParent = TVI_ROOT;
	}
	if (audio != AC_NONE) {
		int ab = 0, aq = 0;
		strcpy(buf, "Audio");
		tvis.item.state |= TVIS_BOLD;
		tvis.hParent = TreeView_InsertItem(hTree,(LPARAM)&tvis);
		tvis.item.state &= ~TVIS_BOLD;
		p = 0;
		switch (audio) {
		case AC_LAMEMP3:
			switch (mcGetPrefValueInt("audioenc.lame.mode")) {
			case 0:
				p = "MP3 (VBR)";
				aq = mcGetPrefValueInt("audioenc.lame.quality");
				break;
			case 1:
				p = "MP3 (VBR)";
				ab = mcGetPrefValueInt("audioenc.lame.bitrate");
				break;
			case 2:
				p = "MP3";
				ab = mcGetPrefValueInt("audioenc.lame.bitrate");
				break;
			}
			break;
		case AC_NEROREF:
			switch (mcGetPrefValueInt("audioenc.nero.format")) {
			case 1:
				p = "LC-AAC";
				break;
			case 2:
				p = "HE-AAC";
				break;
			case 3:
				p = "HE-AAC V2";
				break;
			default:
				p = "AAC";
			}
			if (mcGetPrefValueInt("audioenc.nero.mode") == 0) {
				aq = mcGetPrefValueInt("audioenc.nero.quality");
			} else {
				ab = mcGetPrefValueInt("audioenc.nero.bitrate");
			}
			break;
		case AC_FAAC:
			p = "LC-AAC";
			break;
		case AC_VORBIS:
			p = "Vorbis";
			break;
		case AC_FFMPEG:
			switch (mcGetPrefValueInt("audioenc.ffmpeg.codec")) {
			case 0:	p = "MPEG Layer 2";	break;
			case 2: p = "AC3"; break;
			}
			ab = mcGetPrefValueInt("audioenc.ffmpeg.bitrate");
			break;
		}
		if (p) {
			sprintf(buf, "Format: %s", p);
			TreeView_InsertItem(hTree,(LPARAM)&tvis);
		}
		if (ab) {
			sprintf(buf, "Bitrate: %d Kbps", ab);
			TreeView_InsertItem(hTree,(LPARAM)&tvis);
		} else {
			sprintf(buf, "Quality: %d", aq);
			TreeView_InsertItem(hTree,(LPARAM)&tvis);
		}
		if ((i = mcGetPrefValueInt("audiofilter.resample.samplerate"))) {
			_snprintf(buf, sizeof(buf), "Sample Rate: %d Hz", i);
			TreeView_InsertItem(hTree,(LPARAM)&tvis);
		}
		tvis.hParent = TVI_ROOT;
	}
	if (container != CF_DEFAULT) {
		strcpy(buf, "Container");
		tvis.item.state |= TVIS_BOLD;
		tvis.hParent = TreeView_InsertItem(hTree,(LPARAM)&tvis);
		tvis.item.state &= ~TVIS_BOLD;
		sprintf(buf, "Format: %s", mcGetToolName(TL_CONTAINER, 0, container));
		TreeView_InsertItem(hTree,(LPARAM)&tvis);
		tvis.hParent = TVI_ROOT;
	}
	if (mcGetPrefValueInt("overall.task.mode") == TM_SPLIT) {
		int mode = mcGetPrefValueInt("overall.task.splitMode");
		int val = mcGetPrefValueInt("overall.task.splitParam");
		strcpy(buf, "Split mode");
		tvis.item.state |= TVIS_BOLD;
		tvis.hParent = TreeView_InsertItem(hTree,(LPARAM)&tvis);
		tvis.item.state &= ~TVIS_BOLD;
		if (val) {
			sprintf(buf, "Split into clips of %d %s",
				!mode ? val / 1000 : val,
				!mode ? "secs" : "KB");
			TreeView_InsertItem(hTree,(LPARAM)&tvis);
		}
		tvis.hParent = TVI_ROOT;
	}
	ShowWindow(hTree, SW_SHOW);
}

void CDeviceDialog::ApplySettings()
{
	CString path;
	CString basekey;
	DEV_PLUGIN_INFO* info = GetPluginInfo();
	char *xmldata = info->hdr->xml;
	if (!xmldata) {
		path.Format("%splugins\\%s", apppath, info->pluginFile);
		path.Replace(".dll", ".xml");
		if (FileExist(path.GetBuffer()))
			xmldata = path.GetBuffer();
		
	}
	basekey.Format("/%s/%s", PLUGIN_XML_ROOT, "common");
	mcSetPrefValueXML(xmldata, basekey.GetBuffer());

	for (int i = 0; i < PRESET_CAT; i++) {
		if (!presetInfo[i] ) continue;
		int sel = ComboGetSel(idCombo[i]);
		if (sel < 0) continue;
		if (NotifyPlugin(DEV_EVENT_APPLY, (void*)MAKELONG(i, sel))) continue;
		char* key = strdup(presetInfo[i][sel].key);
		for (char* p = strtok(key, ","); p; p = strtok(0, ",")) {
			basekey.Format("/%s/%s", PLUGIN_XML_ROOT, p);
			mcSetPrefValueXML(xmldata, basekey.GetBuffer());
		}
		free(key);
	}
	RefreshAudioTab();
	RefreshOptionTab();
}
