#include "config.h"
#include <stdio.h>
#include "atlstr.h"
#include "resource.h"
#include "common.h"
#include "discnav.h"
#include "menc.h"
#include "cfilename.h"
#include "httpget.h"

extern CMenc mc;

void EnqueueItem(T_ITEM* item, char* alias = 0);
char* getNetVideoUrl(char* pageurl);

static const char* discPrefix[] = {"dvd://","vcd://","cdda://","cue://","http://","ftp://","rtsp://","mms://","udp://","line://"};
static const char* discDesc[] = {"DVD", "Video CD", "Audio CD", "Cuesheet", "HTTP", "FTP", "RTSP", "MMS", "UDP", "Line-in"};

/*
int GetDVDChapterData(int track, char* discroot, void** data)
{
	DVDOpen pfnDVDOpen;
	ifoOpen pfnIfoOpen;
	ifoClose pfnIfoClose;
	dvd_reader_t *dvd;
	ifo_handle_t *ifohandle;
	int nr;
	HMODULE hDVDNav = LoadLibrary("tools\\libdvdnav-4.dll");
	pfnDVDOpen = (DVDOpen)GetProcAddress(hDVDNav, "DVDOpen");
	pfnIfoOpen = (ifoOpen)GetProcAddress(hDVDNav, "ifoOpen");
	pfnIfoClose = (ifoClose)GetProcAddress(hDVDNav, "ifoClose");
	dvd = (*pfnDVDOpen)(discroot);
	ifohandle = (*pfnIfoOpen)(dvd, track);
	if (ifohandle) {
		if(ifohandle->vtsi_mat) {
			nr = ifohandle->vts_pgcit->nr_of_cells;
			*data = malloc(nr * sizeof(cell_playback_t));
			memcpy(*data, ifohandle->vts_pgcit->cell_playback, nr * sizeof(cell_playback_t));
		} 
		(*pfnIfoClose)(ifohandle);
	}
	FreeLibrary(hDVDNav);
	return nr;
}
*/

void CAddTrackDialog::RefreshDiscInfo()
{
	char discroot[MAX_PATH];
	int track = SendDlgMsg(IDC_TRACK,LB_GETCURSEL,0,0);
	if (!GetDiscRoot(discroot, sizeof(discroot))) return;

	SendDlgMsg(IDC_TYPE,CB_SETCURSEL,tracktype,0);
	SendDlgMsg(IDC_TRACK, LB_RESETCONTENT, 0, 0);
	SendDlgMsg(IDC_TRACK,LB_ADDSTRING,0,(LPARAM)"Whole Disc");
	trackcount = 0;
	switch (tracktype) {
	case TT_CDDA: {
		char buf[16];
		buf[0] = 0;
		trackcount = mcGetDriveTrackCount(SendDlgMsg(IDC_CDROMS,CB_GETCURSEL,0,0));
		disc.SetTrackCount(trackcount);
		if (trackcount > 0) {
			for (int i = 1; i <= trackcount; i++) {
				sprintf(buf, "Track %02d", i);
				SendMessage(GetHandle(IDC_TRACK),LB_ADDSTRING,0,(LPARAM)buf);
			}
		}
		char *p = mcGetDiscInfo(TT_CDDA, SendDlgMsg(IDC_CDROMS,CB_GETCURSEL,0,0), 0);
		SendDlgMsg(IDC_DISCINFO, WM_SETTEXT, 0, (LPARAM)p);
		} break;
	case TT_VCD: {
		HANDLE hFind;
		WIN32_FIND_DATA finddata;
		char buf[MAX_PATH];
		sprintf(buf, "%s\\*.DAT", discroot);
		hFind = FindFirstFile(buf, &finddata);
		if (hFind!=INVALID_HANDLE_VALUE) {
			do {
				if (finddata.cFileName[0]=='.') continue;
				finddata.cFileName[7] = 0;
				SendDlgMsg(IDC_TRACK,LB_ADDSTRING,0,(LPARAM)finddata.cFileName+5);
				trackcount++;
			} while (FindNextFile(hFind,&finddata));
			FindClose(hFind);
		}
		} break;
	case TT_DVD:
		{
			HANDLE hFind;
			WIN32_FIND_DATA finddata;
			strcat(discroot, "\\VTS_*.IFO");
			hFind = FindFirstFile(discroot, &finddata);
			if (hFind!=INVALID_HANDLE_VALUE) {
				do {
					if (finddata.cFileName[0]=='.' || finddata.cFileName[6]!='_') continue;
					finddata.cFileName[6] = 0;
					SendDlgMsg(IDC_TRACK,LB_ADDSTRING,0,(LPARAM)finddata.cFileName+4);
					trackcount++;
				} while (FindNextFile(hFind,&finddata));
				FindClose(hFind);
			}
		} break;
	case TT_CUESHEET: {
		if (cue.LoadCueSheet(discroot)) break;
		SendDlgMsg(IDC_DISCINFO, WM_SETTEXT, 0, (LPARAM)cue.data);
		trackcount = cue.ParseCueSheet();
		CString str;
		if (trackcount > 0) {
			for (int i = 1; i <= trackcount; i++) {
				TRACK_TOKEN* tokens = cue.GetTrackInfo(i, "TITLE");
				str.Format("%02d ", i);
				if (tokens && tokens->value) str += tokens->value;
				SendDlgMsg(IDC_TRACK,LB_ADDSTRING,0,(LPARAM)str.GetBuffer());
			}
		}
		} break;
	}
}

void CAddTrackDialog::RefreshTrackInfo()
{
	char* info;
	int track = SendDlgMsg(IDC_TRACK,LB_GETCURSEL,0,0);
	char discroot[MAX_PATH];
	GetDiscRoot(discroot, sizeof(discroot));
	switch (tracktype) {
	case TT_DVD:
		info = track ? mcGetDiscInfo(TT_DVD, track, discroot) : 0;
		SendDlgMsg(IDC_CHAPTERS, LB_RESETCONTENT, 0, 0);
		SendDlgMsg(IDC_CHAPTERS, LB_ADDSTRING, 0, (LPARAM)"Whole Track");
		if (info) {
			SendDlgMsg(IDC_DISCINFO, WM_SETTEXT, 0, (LPARAM)info);	
			char *p = info;
			for (int i = 1; ; i++) {
				char pattern[16];
				char *s;
				sprintf(pattern, "Cell:  %2d ", i);
				s = strstr(p, pattern);
				if (!s) break;
				s += 7;
				if (*s == ' ') *s = '0';
				p = strchr(s, '@');
				if (p) *(p++) = 0;
				SendDlgMsg(IDC_CHAPTERS, LB_ADDSTRING, 0, (LPARAM)s);
			}
		}
		break;
	case TT_CDDA:
		break;
	default:
		SendDlgMsg(IDC_CHAPTERS, LB_RESETCONTENT, 0, 0);
	}
}

int CAddTrackDialog::GetDiscRoot(char* buf, int bufsize)
{
	char* rootdir;
	int len = SendDlgMsg(IDC_URL,WM_GETTEXT,bufsize,(LPARAM)buf);

	switch (tracktype) {
	case TT_DVD:
		rootdir = "VIDEO_TS";
		break;
	case TT_VCD:
		rootdir = "MPEGAV";
		break;
	case TT_CDDA:
		rootdir = "";
		break;
	default:
		return SendDlgMsg(IDC_URL, WM_GETTEXT, bufsize, (LPARAM)buf);
	}
	if ((len <= 0)) {
		if (SendDlgMsg(IDC_CDROMS,WM_GETTEXT,4,(LPARAM)buf) < 0)
			return 0;
		strcat(buf, rootdir);
		return strlen(buf);
	} else if (len > 3 && buf[len - 1] == '\\') {
		buf[--len] = 0;
		return len;
	}
	return 0;
}

void CAddTrackDialog::Add()
{
	int i;
	char url[240];
	int urllen;
	char discroot[MAX_PATH];
	int seltracks = 0, chaptercount = 0;
	int totalchapters = SendDlgMsg(IDC_CHAPTERS,LB_GETCOUNT,0,0);
	if (!GetDiscRoot(discroot, sizeof(discroot))) return;
	urllen = SendDlgMsg(IDC_URL,WM_GETTEXT,sizeof(url),(LPARAM)url);
	T_ITEM* item;

	for (i = 1; i <= trackcount; i++) {
		if (SendDlgMsg(IDC_TRACK,LB_GETSEL,i,0) > 0) {
			seltracks++;
		}
	}
	for (i = 1; i <= totalchapters; i++) {
		if (SendDlgMsg(IDC_CHAPTERS,LB_GETSEL,i,0) > 0) {
			chaptercount++;
		}
	}

	CString fullpath, alias;
	if (seltracks > 0) {
		for (i = 1; i <= trackcount; i++) {
			if (SendDlgMsg(IDC_TRACK,LB_GETSEL,i,0) <= 0) continue;
			switch (tracktype) {
			case TT_CDDA: {
				fullpath.Format("%sTrack%02d.cda", discroot, i);
				item = mcMakeItem(fullpath.GetBuffer());
				if (!disc.track[i - 1].title.IsEmpty())
					item->info.tags.title = mcStrDup(disc.track[i - 1].title.GetBuffer());
				if (!disc.track[i - 1].artist.IsEmpty())
					item->info.tags.artist = mcStrDup(disc.track[i - 1].artist.GetBuffer());
				if (!disc.genre.IsEmpty())
					item->info.tags.genre = mcStrDup(disc.genre.GetBuffer());
				if (disc.year)
					item->info.tags.year = disc.year;
				item->info.tags.track = i;
				item->info.tags.trackcount = trackcount;
				if (item->info.tags.title) {
					item->info.tags.valid = 1;
					alias.Format("%s - %s", item->info.tags.artist, item->info.tags.title);
				} else {
                    alias.Format("CD Track %d", i);
				}
				EnqueueItem(item, alias.GetBuffer());
				}
				break;
			case TT_CUESHEET: {
				T_ITEM* item = mcMakeItem(cue.file);
				cue.FillCueSheetItem(i, item);
				if (item->info.tags.valid) {
					if (item->info.tags.artist) {
						alias.Format("%d. %s - %s", item->info.track, item->info.tags.artist, item->info.tags.title);
					} else {
						alias = item->info.tags.title;
					}
				} else {
					CFileName fn;
					fn = cue.file;
					alias.Format("Track %d of ", i);
					fn.GetName(alias);
				}
				EnqueueItem(item, alias.GetBuffer());
				} break;
			case TT_DVD:
				if (chaptercount == 0) {
					T_ITEM* item;
					fullpath.Format("%s%d", discPrefix[tracktype], i);
					item = mcMakeItem(fullpath.GetBuffer());
					item->info.track = i;
					item->info.discroot = mcStrDup(discroot);
					alias.Format("DVD Track %d", i);
					EnqueueItem(item, alias.GetBuffer());
				} else {
					for (int j = 1; j <= totalchapters; j++) {
						char chapterinfo[16];
						char* s;
						if (SendDlgMsg(IDC_CHAPTERS,LB_GETSEL,j,0) <= 0) continue;
						fullpath.Format("%s%d", discPrefix[tracktype], i);
						item = mcMakeItem(fullpath.GetBuffer());
						SendDlgMsg(IDC_CHAPTERS, LB_GETTEXT, j, (LPARAM)chapterinfo);
						s = chapterinfo + 3;
						item->info.end = atoi(s) * 3600000 + atoi(s + 3) * 60000 + atoi(s + 6) * 1000 + atoi(s + 9) * 1000 / 30;	//FIXME: assuming 30 fps now
						item->info.track = i;
						item->info.chapter = j;
						item->info.discroot = mcStrDup(discroot);
						alias.Format("DVD Track %d Chapter %d", i, j);
						EnqueueItem(item, alias.GetBuffer());
					}
				}
				break;
			case TT_VCD: {
				HANDLE hFind;
				WIN32_FIND_DATA finddata;
				char buf[MAX_PATH];
				sprintf(buf, "%s\\*%d.DAT", discroot, i);
				hFind = FindFirstFile(buf, &finddata);
				if (hFind!=INVALID_HANDLE_VALUE) {
					fullpath.Format("%s\\%s", discroot, finddata.cFileName);
					item = mcMakeItem(fullpath.GetBuffer());
					alias.Format("VCD Track %d", i);
					FindClose(hFind);
					EnqueueItem(item, alias.GetBuffer());
				}
				} break;
			default:
				fullpath.Format("%s%d", discPrefix[tracktype], i + 1);
				item = mcMakeItem(fullpath.GetBuffer());
				EnqueueItem(item, fullpath.GetBuffer());
			}
		}
	} else {
		switch (tracktype) {
		case TT_DVD:
			fullpath.Format("%s", discPrefix[tracktype]);
			item = mcMakeItem(fullpath.GetBuffer());
			item->info.discroot = mcStrDup(discroot);
			EnqueueItem(item, "DVD");
			break;
		case TT_CUESHEET: {
			CFileName cuefile;
			cuefile = cue.file;
			cuefile.GetName(alias);
			T_ITEM* item = mcMakeItem(cue.file);
			cue.FillCueSheetItem(0, item);
			EnqueueItem(item, alias.GetBuffer());
			} break;
		default: {
			char *p = getNetVideoUrl(url);
			if (p) {
				T_ITEM* item = mcMakeItem(p);
				EnqueueItem(item);
				free(p);
			} else {
				MsgBox(IDS_URL_ERROR);
			}
			} break;
		}
	}
}

bool CAddTrackDialog::onClick(int dlgid)
{
	switch (dlgid) {
	case IDOK:
		Add();
		break;
	case IDCANCEL:
		Destroy();
		break;
	case IDC_REFRESH:
		RefreshDiscInfo();
		break;
	case IDC_QUERY: {
		CCddb cddb;
		CString str;
		cddb.QueryDisc(SendDlgMsg(IDC_CDROMS,CB_GETCURSEL,0,0));
		SendDlgMsg(IDC_CHAPTERS, LB_RESETCONTENT,0, 0);
		for (int i = 0; !cddb.GetQueryOptions(i, str); i++) {
			SendDlgMsg(IDC_CHAPTERS, LB_ADDSTRING,0, (LPARAM)str.GetBuffer());
		}
		if (cddb.QueryTrackNames() == -1) break;
		SendDlgMsg(IDC_DISCINFO, WM_SETTEXT, 0, (LPARAM)cddb.result.GetBuffer());

		int curpos = 0;
		while ((str = cddb.result.Tokenize("\r\n", curpos)) != "") {
			if (str.GetAt(0) == '#') continue;
			if (str.Left(6) == "TTITLE") {
				int trackno = atoi(str.Mid(6).GetBuffer());
				int pos = str.Find('=');
				disc.track[trackno].title = str.Mid(pos + 1);
				int sep;
				sep = disc.track[trackno].title.Find(" / ");
				if (sep != -1) {
					disc.track[trackno].artist = disc.track[trackno].title.Left(sep);
					disc.track[trackno].title = disc.track[trackno].title.Mid(sep + 3);
				} else {
					disc.track[trackno].artist = disc.artist;
				}
			} else if (str.Left(6) == "DTITLE") {
				int sep;
				disc.title = str.Mid(7);
				sep = disc.title.Find(" / ");
				if (sep != -1) {
					disc.artist = disc.title.Left(sep);
					disc.title = disc.title.Mid(sep + 3);
				}
			} else if (str.Left(6) == "DGENRE") {
				disc.genre = str.Mid(7);
			} else if (str.Left(5) == "DYEAR") {
				disc.year = atoi(str.Mid(6));
			}
		}
		SendDlgMsg(IDC_TRACK, LB_RESETCONTENT, 0, 0);
		SendDlgMsg(IDC_TRACK,LB_ADDSTRING,0,(LPARAM)"All");
		for (int i = 0; i < trackcount; i++) {
			CString str;
			str.Format("%s - %s", disc.track[i].artist, disc.track[i].title);
			SendDlgMsg(IDC_TRACK,LB_ADDSTRING,0,(LPARAM)str.GetBuffer());
		}
		} break;
	case IDC_BROWSE: {
		char buf[MAX_PATH];
		switch (tracktype) {
		case TT_DVD:
		case TT_VCD:
		case TT_CDDA:
			if (BrowseForPath(buf)) {
				SendDlgMsg(IDC_URL, WM_SETTEXT, 0, (LPARAM)buf);
			}
			break;
		case TT_CUESHEET:{
			OPENFILENAME ofn;
			memset(&ofn, 0, sizeof(ofn));
			buf[0]=0;
			ofn.lpstrFile = buf;
			ofn.nMaxFile = sizeof(buf);
			ofn.lpstrFilter = "CueSheet\0*.cue\0";
			ofn.lpstrTitle = "Load a cuesheet file";
			if (!ShowFileDialog(0,&ofn)) {
				SendDlgMsg(IDC_URL, WM_SETTEXT, 0, (LPARAM)ofn.lpstrFile);
			}
			}
		}
		RefreshDiscInfo();
		} break;
	default:
		return false;
	}
	return true;
}

bool CAddTrackDialog::onSelChange(int dlgid)
{
	switch (dlgid) {
	case IDC_TYPE:
		tracktype = SendDlgMsg(IDC_TYPE,CB_GETCURSEL,0,0);
	case IDC_CDROMS:
		RefreshDiscInfo();
		break;
	case IDC_TRACK:
		RefreshTrackInfo();
		break;
	default:
		return false;
	}
	return true;
}

bool CAddTrackDialog::onCreate()
{
	{
		DWORD drv=GetLogicalDrives();
		DWORD c = 8;
		char l = 'D';
		int i , n = mcGetDriveCount();
		for (i = 0; i < n; c <<= 1) {
			if (drv & c) {
				char buf[128];
				sprintf(buf,"%c:\\", l++);
				if (GetDriveType(buf) == DRIVE_CDROM) {
					SendDlgMsg(IDC_CDROMS,CB_ADDSTRING,0,(LPARAM)buf);
					i++;
				}
			}
		}
		SendDlgMsg(IDC_CDROMS,CB_SETCURSEL,0,0);
	}

	for (int i = 0; i < TT_COUNT; i++) {
		SendDlgMsg(IDC_TYPE,CB_ADDSTRING,0,(LPARAM)discDesc[i]);
	}
	SendDlgMsg(IDC_TYPE,CB_SETCURSEL,0,0);
	RefreshDiscInfo();
	return true;
}

bool CAddTrackDialog::onDestroy()
{
	return true;
}

void CCueSheet::Purge()
{
	if (trackcount == 0) return;
	if (data) {
		free(data);
		data = 0;
	}
	for (int i = 0; i <= trackcount; i++)
	{
		if (tracks[i].tokens) free(tracks[i].tokens);
	}
	free(tracks);
	trackcount = 0;
}

int CCueSheet::LoadCueSheet(char* filename)
{
	Purge();
	if (LoadTextFile(filename, &data, 0) < 0) return -1;
	strcpy(file, filename);
	return 0;
}

int CCueSheet::FillCueSheetItem(int trackno, T_ITEM* item)
{
	CString alias, args;
	TRACK_TOKEN* title;
	TRACK_TOKEN* artist;

	if (trackno > trackcount) return -1;

	title = GetTrackInfo(trackno, "TITLE");
	artist = GetTrackInfo(trackno, "PERFORMER");

	if (!item->filename)
		item->filename = mcStrDup(file);
	if (trackno > 0) {
		item->info.duration = tracks[trackno].end - tracks[trackno].begin;
		item->info.filesize = 0;	//FIXME: assuming CD quality
		item->info.begin = tracks[trackno].begin;
		item->info.end = tracks[trackno].end;
		if (!artist) artist = GetTrackInfo(0, "PERFORMER");
		item->info.track = trackno;
		item->info.tags.track = trackno;
	}
	if (title) {
		item->info.tags.valid = 1;
		item->info.tags.title = mcStrDup(title->value);
	}
	if (artist) item->info.tags.artist = mcStrDup(artist->value);
	return 0;
}

int CCueSheet::ParseCueSheet()
{
	char *p, *nl;
	int trackno;

	for (p = data; ; p++)
	{
		while (*p == ' ') p++;
		if (!strncmp(p, "TRACK ", 6)) {
			trackno = atoi(p + 6);
			if (trackno > trackcount)
				trackcount = trackno;
		}
		p = strchr(p, '\n');
		if (!p) break;
	}
	tracks = (CUE_TRACK_INFO*)calloc(trackcount + 1, sizeof(CUE_TRACK_INFO));

	trackno = 0;
	CUE_TRACK_INFO* track = tracks;
	for (p = data; p ; p = nl)
	{
		int i;
		nl = strchr(p, '\n');
		if (nl)
		{
			if (*(nl - 1) == '\r')
				*(nl - 1) =0;
			else
				*nl = 0;
			nl++;
		}
		while (*p == ' ') p++;
		if (!*p) continue;
		i = track->valuecount++;
		track->tokens = (TRACK_TOKEN*)realloc(track->tokens, track->valuecount * sizeof(TRACK_TOKEN));
		track->tokens[i].name = p;
		track->tokens[i].value = 0;
		track->tokens[i].subvalue = 0;
		p = strchr(p, ' ');
		if (p) {
			*(p++) = 0;
			if (*p == '\"') {
				track->tokens[i].value = ++p;
				p = strchr(p, '\"');
				*(p++) = 0;
				if (*p == ' ') track->tokens[i].subvalue = p + 1;
			} else {
				track->tokens[i].value = p;
				p = strchr(p, ' ');
				if (p) track->tokens[i].subvalue = p + 1;
			}
		}
		
		if (!strcmp(track->tokens[i].name, "FILE")) {
			char *t;
			if (track->tokens[i].value[1] == ':') {
				t = file;
			} else {
				t = strrchr(file, '\\');
				if (!t++) t = strchr(file, 0);
			}
			strcpy(t, track->tokens[i].value);
		} else if (!strcmp(track->tokens[i].name, "INDEX")) {
			if (track->tokens[i].subvalue) {
				char *t = track->tokens[i].subvalue;
				unsigned int n = atoi(t) * 60000 + atoi(t + 3) * 1000 + + atoi(t + 6) * 10;
				if (n > track->begin)
					track->begin = n;
				if (trackno > 1 && (track - 1)->end == 0)
					(track - 1)->end = track->begin;
			}
		} else if (!strcmp(track->tokens[i].name,"TRACK")) {
			trackno = atoi(track->tokens[i].value);
			track = &tracks[trackno];
		}
	}
	return trackcount;
}

TRACK_TOKEN* CCueSheet::GetTrackInfo(int trackno, char* name)
{
	if (trackno <= trackcount) {
		for (int i = 0; i < tracks[trackno].valuecount; i++) {
			if (!strcmp(name, tracks[trackno].tokens[i].name))
				return &tracks[trackno].tokens[i];
		}
	}
	return 0;
}

int CCddb::QueryDisc(int cdrom)
{
	CString querystr = "cmd=";
	char *p = mcGetDiscInfo(TT_CDDA, cdrom, 0);
	if (!p) return -1;
	querystr += p;
	querystr += "&hello=aguser+audiograbber+audiograbber+1.83.01&proto=5\r\n";
	querystr.Replace(' ', '+');
	httpRequest param;
	memset(&param, 0, sizeof(param));
	param.method = HM_POST;
	param.url = "http://www.freedb.org/~cddb/cddb.cgi";
	param.postData = querystr.GetBuffer();
	param.iPostDataSize = querystr.GetLength();
	if (httpGet(&param)) return -1;
	result = param.buffer;
	if (param.buffer) free(param.buffer);
	return 0;
}

int CCddb::GetQueryOptions(int opt, CString& str)
{
	int curpos = 0;
	CString token;
	token = result.Tokenize("\r\n",curpos);
	if (token.Find("Found") == -1) return -1;
	for (int i = 0; i <= opt; i++) {
		str = result.Tokenize("\r\n",curpos);
		if (str == ".") return -1;
	}
	return 0;
}

int CCddb::QueryTrackNames()
{
	int curpos = 0;
	CString token;

	token = result.Tokenize("\r\n",curpos);
	if (token.Find("Found") == -1) return -1;
	while ((token = result.Tokenize("\r\n",curpos)) != "") {
		int pos = token.Find(' ');
		if (pos == -1) return -1;
		pos = token.Find(' ', pos + 1);
		if (pos == -1) return -1;
		CString querystr;
		querystr = "cmd=cddb read " + token.Left(pos) + "&hello=aguser+audiograbber+audiograbber+1.83.01&proto=5\r\n";
		querystr.Replace(' ', '+');

		httpRequest param;
		memset(&param, 0, sizeof(param));
		param.method = HM_POST;
		param.url = "http://www.freedb.org/~cddb/cddb.cgi";
		param.postData = querystr.GetBuffer();
		param.iPostDataSize = querystr.GetLength();
		if (httpGet(&param)) return -1;
		result = param.buffer;
		if (param.buffer) free(param.buffer);
	}
	return 0;
}

void CDiscInfo::SetTrackCount(int count)
{
	if (trackcount) {
		delete [] track;
	}
	trackcount = count;
	track = new TRACK_INFO[count];
}