/*******************************************************************
* MediaCoder - The universal audio/video transcoder
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*******************************************************************/

#ifndef MENC_H
#define MENC_H

#define VER_MAJOR 0
#define VER_MINOR 5
#define VER_REV 0

#define VERSION (VER_MAJOR * 100 + VER_MINOR * 10 + VER_REV)

#define CFGFILE "prefs.xml"
#define LISTFILE "mediacoder.m3u"
#define APPNAME "MediaCoder"
#define STRCMP(s1, s2) (*(LPDWORD)s1 != *(LPDWORD)s1 || strcmp(s1, s2))
#define AUDIODIALOG AudioDialogs[iAudCodecDlg]

typedef enum {
	UI_SIMPLE = 0,
	UI_DEVICE,
	UI_NORMAL,
	UI_EXPERT,
} UI_MODE;

void MsgBoxStr(UINT uID);
T_ITEM* GetSelectedItem(int defidx = 0);
int StartTranscode(int mode = 0);
void ListEnums(char* key, HWND hCtrl, UINT msg);
void RefreshGui();
void RefreshOptionTab();
void RefreshAudioTab();
int GetFileProgress();
int ShowWeb(char* path);
int ShowSetupWizard(bool showNotice = false);
void EnqueueFile(char* filename, char* alias = 0);
void ShowCropper(CWindow* parent, float aspectratio = 0);

#define PATH_COL 3

typedef enum {
	VIEW_LOG = 0,
	VIEW_CONSOLE,
	VIEW_AUDIO,
	VIEW_VIDEO,
	VIEW_CONTAINER,
} VIEWS;

enum {
	LC_FILENAME = 0, LC_SIZE, LC_TYPE, LC_STATE, LC_FORMAT, LC_FULLPATH, LC_COUNT,
};

class CMenc:public CWindow
{
public:
	int Init();
	int AddItem(T_ITEM* item, char* aliasstr = 0);
	int AddMultipleFiles(char* filenames);
	void ShowConsole(int fShow);
	void UpdateTitle(char* text);
	void UpdateStatusBar(LPSTR lpszStatusString, WORD partNumber, WORD displayFlags);
	void UpdateTabbedView();
	CMenc();
	int GenCommandLines();
	void ShowAudioDialog(int idx);
	void ShowSubDialog(int idx);
	void ShowState(int state);
	void Refresh();
	void ShowStatsWindow(bool fShow);
	void SetInitialPos();
	void SwitchUIMode();
	int SaveQueue(char* filename);
	int LoadQueue(char* filename);
	void RevertPref();
	bool onMove();
	bool onClick(int dlgid);
	CListView list;
	bool fConsole;
	HWND hInfoBox;
private:
	bool onNotify(LPNMHDR pnmh);
	bool onSelChange(int dlgid);
	bool onCommand(int dlgid, int code);
	bool onCreate();
	bool onClose();
	bool onSize(int width, int height);
	bool onShow(bool fShow);
	int SetFormatMode(int mode);
	int CheckFileExistence(char* filename);
	int GetFileFromList(int idx, char* buf);
	int EnumDir(char* dir);
	void AddFolder(char* dir);
	char *version;
	HWND hWndStatusbar;
	CWindow opttab;
	CWindow audiotab;
};

typedef enum {
	CTRL_TEXT = 0,
	CTRL_COMBO,
	CTRL_BUTTON,
	CTRL_CHECK,
	CTRL_RADIO,
	CTRL_SLIDER,
} CTRL_TYPE;

typedef struct {
	int ctrlid;
	char* prefkey;
	int index;
	CTRL_TYPE ctrltype;
} PREF_CTRL;

class COptionDialog:public CWindow
{
public:
	COptionDialog();
	~COptionDialog() { Destroy(); }
	void RefreshDialog(int activectrl = 0);
	virtual void Refresh() {}
	virtual char* GetName() { return 0; }
	int dialogid;
protected:
	bool onCreate();
	bool onCommand(int dlgid, int code);
	bool onScroll(int dlgid, int code);
	virtual bool onCreate2() { return true; }
	virtual bool onCommand2(int dlgid, int code) { return false; }
	virtual bool onScroll2(int dlgid, int code) { return false; }
	PREF_CTRL* ctrls;
	int refreshing;
};

class CVideoWindow:public CWindow
{
public:
	CVideoWindow();
	void Open(HWND hwndParent, int windowFlags = 0);
	bool movable;
private:
	bool onMouseMove(DWORD pos, int vkey);
	HWND hParent;
	int mx;
	int my;
	HCURSOR hCursor1;
	HCURSOR hCursor2;
};

#endif
