/*************************************************************************
* MediaCoder - The universal audio/video transcoder
*
* Copyright (c) 2005-06 Stanley Huang <reachme@citiz.net>
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. The name of the author may not be used to endorse or promote products
*    derived from this software without specific prior permission.
*
* THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*************************************************************************/

#include "config.h"
#include "resource.h"
#include "atlstr.h"
#include "menc.h"
#include "optiondialogs.h"

void ShowCropper(CWindow* wnd, float aspectratio);

void CPsp::Open(CWindow* parent)
{
	if (hwnd) return;
	state = 0;
	CreateDlg(0, IDD_PSP, parent);
	SendDlgMsg(IDC_PLAY,BM_SETIMAGE,IMAGE_BITMAP,(LPARAM)(LoadBitmap(hInstance,MAKEINTRESOURCE(IDB_PSP_PLAY))));
	SendDlgMsg(IDC_START,BM_SETIMAGE,IMAGE_BITMAP,(LPARAM)(LoadBitmap(hInstance,MAKEINTRESOURCE(IDB_PSP_START))));
	SendDlgMsg(IDC_CROP,BM_SETIMAGE,IMAGE_BITMAP,(LPARAM)(LoadBitmap(hInstance,MAKEINTRESOURCE(IDB_PSP_CROP))));
	SendDlgMsg(IDC_PREVIEW,BM_SETIMAGE,IMAGE_BITMAP,(LPARAM)(LoadBitmap(hInstance,MAKEINTRESOURCE(IDB_PSP_PREVIEW))));
	SetItemChecked(IDC_QUALITY_MODE);
	SetItemChecked(IDC_RADIO2);
	SetItemChecked(IDC_RADIO4);
	SetItemChecked(IDC_RADIO9);
	SetItemChecked(IDC_RADIO15);

	char* presets[] = {"Fast", "Normal", "Quality", 0};
	for (int i = 0; presets[i]; i++) {
		SendDlgMsg(IDC_PRESET, CB_ADDSTRING, 0, (LPARAM)presets[i]);
	}
	ComboSetSel(IDC_PRESET, 1);
	ApplySettings();
	Refresh();
	Show();
	SetTimer(hwnd, 0, 1000, 0);
}

void CPsp::Refresh()
{
}

void CPsp::Start()
{
	if (pspdrv) {
		TCHAR buf[32];
		int len = sprintf(buf, "%c:\\PSP\\VIDEO", pspdrv);
		TCHAR* p = mcGetPrefValueStr("overall.task.destdir");
		if (strncmp(p, buf, len)) {
			CString str;
			str.Format(IDS_PSP_DIRECT, buf);
			if (Ask(str.GetBuffer(), MB_YESNO) == IDYES) {
				sprintf(p, "%s\\", buf);
				CreateDirectory(buf, 0);
				RefreshGui();
			}
		}
	}
	ApplySettings();
	StartTranscode();
}

bool CPsp::onClick(int dlgid)
{
	switch (dlgid) {
	case IDC_PLAY: 
		if (state == 0) {
			HWND hVideo = 0;
			T_ITEM* item = GetSelectedItem();
			if (item) {
				mcGetMediaInfo(item);
				ApplySettings();
				if (mcPlayerStart(item, GetHandle(IDC_SCREEN), &hVideo, 0, "-nokeepaspect")) break;
				if (hVideo) {
					state = 1;
					MoveWindow(hVideo, 0, 0, 480, 272, 1);
					SendDlgMsg(IDC_POS,TBM_SETRANGE,FALSE,MAKELPARAM(0,item->info.duration / 1000));
					SendDlgMsg(IDC_POS, TBM_SETPOS, FALSE, 0);
					EnableCtrl(IDC_POS);
				}
			} else {
				MsgBox(IDS_NO_FILE);
			}
		} else if (state == 1) {
			HideCtrl(IDC_POS);
			state = 0;
			mcPlayerStop();
		}
		break;
	case IDC_START:
		Start();
		break;
	case IDC_CROP: {
		T_ITEM* item = GetSelectedItem();
		float aspect = 0;
		if (item && !mcGetMediaInfo(item) && !item->target.crop.x && !item->target.crop.y) {
			int h = mcGetPrefValueInt("videofilter.scale.width");
			if (h) {
				aspect = (float)mcGetPrefValueInt("videofilter.scale.height") / h;
			}
		}
		ShowCropper(this, aspect);
		} break;
	case IDC_PREVIEW:
		if (state == 0) {
			HWND hVideo = 0;
			if (!mcPlayerStart(GetSelectedItem(), GetHandle(IDC_SCREEN), &hVideo, PF_PREVIEW, 0)) {
				if (hVideo) {
					state = 1;
					MoveWindow(hVideo, 0, 0, 480, 272, 1);
				}
			}
		} else {
			state = 0;
			mcPlayerStop();
		}
		break;
	}
	ApplySettings();
	RefreshOptionTab();
	RefreshAudioTab();
	return true;
}

bool CPsp::onScroll(int dlgid, int code)
{
	int pos = SendDlgMsg(IDC_POS,TBM_GETPOS,0,0);
	char buf[32];
	sprintf(buf, "seek %d 2", pos);
	mcPlayerCommand(buf);
	return true;
}

char CPsp::CheckPSP()
{
	static DWORD dm = 0;
	char drv;
	DWORD d = GetLogicalDrives();
	if (d == dm) return 0;
	dm = d;
	drv = 'Z';
	for (DWORD m = (1 << 25); drv > 'C'; m >>= 1, drv--) {
		if (d & m) {
			TCHAR buf[32];
			sprintf(buf, "%c:\\", drv);
			if (GetDriveType(buf) == DRIVE_REMOVABLE) {
				DWORD ret;
				strcat(buf, "PSP");
				ret = GetFileAttributes(buf);
				if (ret != INVALID_FILE_ATTRIBUTES && (ret & FILE_ATTRIBUTE_DIRECTORY)) {
					return drv;
				}
			}
		}
	}
	return -1;
}

bool CPsp::onTimer(int id)
{
	char drv = CheckPSP();
	if (drv < 0) {
		CString str;
		str.Format(IDS_PSP_UNDETECTED);
		SetText(IDC_INFO, str.GetBuffer());
		pspdrv = 0;
	} else if (drv > 0) {
		CString str;
		str.Format(IDS_PSP_DETECTED, drv);
		SetText(IDC_INFO, str.GetBuffer());
		pspdrv = drv;
	}
	if (state == 1) {
		int pos = SendDlgMsg(IDC_POS,TBM_GETPOS,0,0);
		if (state == 0 || mcPlayerCommand("get_time_pos")) {
			HideCtrl(IDC_POS);
			state = 0;
		} else {
			SendDlgMsg(IDC_POS, TBM_SETPOS, TRUE, pos + 1);
		}
	}
	return true;
}

bool CPsp::onClose()
{
	KillTimer(hwnd, 0);
	if (state == 1) {
		onClick(IDC_PLAY);
	}
	mcPlayerStop();
	EndDialog(hwnd, 0);
	return true;
}

void CPsp::ApplySettings()
{
	int w;
	int h;
	float fps;
	int videores;
	int videomode;
	int audiomode;
	int preset = ComboGetSel(IDC_PRESET);
	{
		int radiobox[] = {IDC_RADIO1, IDC_RADIO2, IDC_RADIO3, 0};
		videomode = GetRadioBoxValue(radiobox);
	}	
	{
		int radiobox[] = {IDC_RADIO4, IDC_RADIO5, IDC_RADIO6, IDC_RADIO7, 0};
		char buf[16];
		videores = GetRadioBoxValue(radiobox);
		SendDlgMsg(radiobox[videores], WM_GETTEXT, sizeof(buf), (LPARAM)buf);
		w = atoi(buf);
		h = atoi(strchr(buf, 'x') + 1);
	}	
	{
		int radiobox[] = {IDC_RADIO9, IDC_RADIO10, IDC_RADIO11, IDC_RADIO12, IDC_RADIO13, 0};
		int i = GetRadioBoxValue(radiobox);
		char buf[16];
		if (i) {
			SendDlgMsg(radiobox[i], WM_GETTEXT, sizeof(buf), (LPARAM)buf);
			fps = (float)atoi(buf);
		} else {
			fps = 0;
		}
	}	
	{
		int radiobox[] = {IDC_RADIO14, IDC_RADIO15, IDC_RADIO16, 0};
		audiomode = GetRadioBoxValue(radiobox);
	}

	mcSetPrefValueInt("videofilter.scale.enabled", 1);
	mcSetPrefValueInt("videofilter.scale.preset", 0);
	mcSetPrefValueInt("videofilter.scale.width", w);
	mcSetPrefValueInt("videofilter.scale.height", h);
	mcSetPrefValueInt("audiofilter.resample.samplerate", 44100);
	mcSetPrefValueInt("audiofilter.resample.downSamplingOnly", 1);

	static int audioquant[] = {9, 7, 6};
	static int videobitrate[4][3] = {{300, 600, 800}, {250, 450, 600}, {200, 350, 500}, {300, 450, 600}};

	mcSetPrefValueInt("audioenc.lame.quality", audioquant[audiomode]);
	mcSetPrefValueInt("audioenc.lame.aq", 1);

	mcSetPrefValueFloat("overall.video.fps", fps);
	mcSetPrefValueInt("overall.video.bitrate", videobitrate[videores][videomode]);
	mcSetPrefValueInt("overall.video.format", VC_XVID);
	mcSetPrefValueInt("overall.video.mode", 0);
	mcSetPrefValueInt("overall.video.container", CF_PMP);
	mcSetPrefValueInt("overall.audio.encoder", AC_LAMEMP3);

	mcSetPrefValueInt("videoenc.xvid.profile", 0);
	mcSetPrefValueInt("videoenc.xvid.cartoon", IsItemChecked(IDC_CARTOON) ? 1 : 0);
	mcSetPrefValueInt("videoenc.xvid.bframe", 0);
	mcSetPrefValueInt("videoenc.xvid.chroma_me", 1);
	mcSetPrefValueInt("videoenc.xvid.chroma_opt", 1);
	mcSetPrefValueInt("videoenc.xvid.hqac", 1);
	mcSetPrefValueInt("videoenc.xvid.gmc", 0);
	mcSetPrefValueInt("videoenc.xvid.trellis", 1);
	mcSetPrefValueInt("videoenc.xvid.vhq", 0);
	switch (preset) {
	case 0: // fast mode
		mcSetPrefValueInt("videoenc.xvid.hqac", 0);
		mcSetPrefValueInt("videoenc.xvid.trellis", 0);
		break;
	case 2: // high quality mode
        mcSetPrefValueInt("overall.video.mode", 2);
		mcSetPrefValueInt("audioenc.lame.aq", 0);
		mcSetPrefValueInt("videoenc.xvid.meQuality", 6);
		mcSetPrefValueInt("videoenc.xvid.vhq", 4);
		mcSetPrefValueInt("videoenc.xvid.gmc", 1);
		break;
	}

}
